#################
#               #
#  M A G I C C  #
#               #
#################

set Version "4.1"
set Copyright "March 2003"
set RunMode "UNDP"

source loads.tcl
source colours.tcl
source fonts.tcl
source values.tcl
source dialogs.tcl
source refer.tcl
source emis.tcl
source run.tcl
source files.tcl
source graphs.tcl
source print.tcl
source errbox.tcl
source html_lib.tcl
source help.tcl
source scengen.tcl
source caveat.tcl

switch $tcl_platform(platform) {
  windows { 
    source wprint.tcl
  }
}

proc MainScreen {} {
  global RunMode tcl_platform TitleBanner
  wm title . $TitleBanner
  set menubar [frame .menubar -bd 1 -relief raised]
  pack $menubar -side top -fill x -expand on -anchor n
  set mb [menubutton $menubar.file -text File -menu $menubar.file.menu]
  pack $mb -side left
  set m [menu $mb.menu -tearoff 0]
  if {$RunMode=="Full"} {
    $m add command -label Load 
#     -command {FSBox "Load MAGICC parameters" "*.msf" }
    $m add command -label Save
    $m add command -label "Save As..."
    $m add separator
  }
#  $m add command -label Exit -command CleanExit
  $m add command -label Exit -command exit

  set mb [menubutton $menubar.edit -text Edit -menu $menubar.edit.menu]
  pack $mb -side left
  set m [menu $mb.menu -tearoff 0]
  $m add command -label "Emissions Scenarios" -command { EmissionsProfiles }

  if {$RunMode=="UNEP"||$RunMode=="UNDP"} {
    $m add command -label "Model Parameters" -command { UNEPparams        }
  }

  if {$RunMode=="Full"} {
    $m add command -label "Gas Cycle models"   -command { GasCycleParams    }
    $m add command -label "Climate model"      -command { ClimateParams     }
    $m add command -label "Sea Level models"   -command { SeaLevelParams    }
  }

  $m add command -label "Output Years"       -command { OutputParams      }

  set mb [menubutton $menubar.run -text Run -menu $menubar.run.menu ]
  pack $mb -side left
  set m [menu $mb.menu -tearoff 0]
  $m add command -label "Run model" -command { RunModel }

  set mb [menubutton $menubar.view -text View -menu $menubar.view.menu]
  pack $mb -side left
  set m [menu $mb.menu -tearoff 0]
  $m add command -label "Graphs:"
  $m add command -label "  Emissions"               -command EmisGraphs
  $m add command -label "  Concentrations"          -command ConcGraphs
  $m add command -label "  Radiative forcing"       -command ForcGraphs
  $m add command -label "  Temperature & Sea-Level" -command TslGraphs
  $m add separator
  $m add command -label "Reports:"
  $m add command -label "  User Policy" -command {ViewFile report.up "User/Policy report file"}
  $m add command -label "  Default Policy" -command {ViewFile report.dp "Default/Policy report file"}
  $m add command -label "  User Reference" -command {ViewFile report.ur "User/Reference report file"}
  $m add command -label "  Default Reference" -command {ViewFile report.dr "Default/Reference report file"}

  if [SCENGENinstalled] {
    set mb [menubutton $menubar.sg -text SCENGEN -menu $menubar.sg.menu ]
    pack $mb -side left
    set m [menu $mb.menu -tearoff 0]
    $m add command -label "Run SCENGEN" -command { RunSCENGEN }
    if {$tcl_platform(platform)=="unix"} {
      $m add command -label "Use output (via xterm)" -command { XtermSCENGEN }
    }
  }

  set mb [menubutton $menubar.help -text Help -menu $menubar.help.menu]
  pack $mb -side right
  set m [menu $mb.menu -tearoff 0]
  $m add command -label "Index" -command { Help index.htm }
  $m add separator
  $m add command -label "General Information"              -command { Help general.htm }
  $m add command -label "Scientific Information"           -command { Help science.htm }
  $m add command -label "Emissions Scenarios"              -command { Help profile.htm }
#  $m add command -label "Editing Emissions Scenarios"      -command { Help edemis.htm  }
  $m add command -label "Model Parameters"                 -command { Help modpar.htm  }
  $m add command -label "Output Years"                     -command { Help outyear.htm }
  $m add command -label "Emissions Graphs"                 -command { Help gremis.htm  }
  $m add command -label "Concentrations Graphs"            -command { Help grconc.htm  }
  $m add command -label "Forcing Graphs"                   -command { Help grforc.htm  }
  $m add command -label "Temperature and Sea Level Graphs" -command { Help grtsl.htm   }
  $m add command -label "Reporting"                        -command { Help report.htm  }
#  $m add command -label References -command { HelpReferences }
#  $m add separator
#  $m add command -label About -command { About }

  set mb [menubutton $menubar.caveat -text ReadMe -menu $menubar.caveat.menu ]
  pack $mb -side right
  set m [menu $mb.menu -tearoff 0]
  $m add command -label "View ReadMe" -command { ViewCaveat }

  AboutContent {}
  
  pack [frame .padding] -side bottom -fill y -expand on
}

proc XtermSCENGEN {} {
  set wd [pwd]
#  cd ../scengen/output
  cd ../engine
  exec xterm &
  cd $wd
}


#set File(policy) is92c
#set File(reference) is92a
set File(policy) B2-MES
set File(reference) A1B-AIM

set TitleBanner "MAGICC $Version"

SetInitialValues Default
SetInitialValues User
MainScreen
