# dialogs.tcl - various dialogs for MAGICC

proc CentreWindow {win} {
  update
  set w  [winfo width $win]
  set h  [winfo height $win]
  set sw [winfo screenwidth $win]
  set sh [winfo screenheight $win]
  set x [expr ($sw-$w)/2]
  set y [expr ($sh-$h)/2]
#puts "window = $w x $h\nscreen = $sw x $sh\nposn = $x $y\n\n"
  wm geometry $win +$x+$y
}

proc About {} {
  toplevel .about
  wm title .about "About MAGICC"
  AboutContent .about
  button .about.okay -text OK -command {destroy .about}
  pack .about.okay -side top -pady 10
  grab .about
}  

proc AboutContent {tl} {
    global Version RunMode Copyright
    label $tl.logo -bitmap @magicc.xbm
    pack $tl.logo -side top -pady 5
    set f [frame $tl.f1]
    pack $f -side top
#    label $f.l -bitmap @magicc.xbm
    label $f.l -bitmap @ncar.xbm
    pack $f.l -side left -padx 5
    set t [frame $f.t]
    pack $t -side left
    label $t.t1 -text "Model for the Assessment of"
    label $t.t2 -text "Greenhouse-gas Induced Climate Change"
    pack $t.t1 $t.t2 -side top -padx 5
    label $f.r -bitmap @cru.xbm
    pack $f.r -side right -padx 5
    label $tl.tar -text "As used in the IPCC Third Assessment Report"
    pack $tl.tar -side top -pady 10
    
    label $tl.ver  -text "Version $Version"
    label $tl.by1 -text \
	    "Concept and Scientific Programming: T.M.L. Wigley, S.C.B. Raper"
    label $tl.by2 -text \
	    "Design: T.M.L. Wigley, M. Salmon, M. Hulme, S.C.B. Raper"
    label $tl.by3 -text "User Interface: M. Salmon, S. McGinnis"
#  label $tl.copy -text "\xa9 Copyright ${Copyright} the Climatic Research Unit"
#  pack $tl.ver $tl.by1 $tl.by2 $tl.by3 $tl.copy -side top -pady 5
  pack $tl.ver $tl.by1 $tl.by2 $tl.by3 -side top -pady 5
}

proc CleanExit {} {
  set tl [toplevel .qyexit]
  wm title $tl "Exit MAGICC"
  label $tl.l1 -text "Are you sure you want\nto exit MAGICC?"
  pack $tl.l1 -side top -pady 10 -padx 10
  set f [frame $tl.buttons -bd 10]
  pack $f -side top -fill x
  button $f.ok -text OK -command {exit}
  button $f.cancel -text Cancel -command "destroy $tl"
  pack $f.ok -side left
  pack $f.cancel -side right
  grab $tl
}

proc VarBox { frm var lbl } {
  global Default User

  set f [frame $frm]
  pack $f -side top -anchor w
  button $f.b -width 5 -text $Default($var) -command "set User($var) \$Default($var)"
  entry $f.e -width 5 -bg white -relief sunken -textvariable User($var)
  label $f.l -text $lbl
  pack $f.b $f.e $f.l -side left
}

proc OutputParams {} {
  global TitleBanner
  if {[winfo exists .outparam]} {
    focus .outparam
    return
  }
  set tl [toplevel .outparam]
  wm title $tl "$TitleBanner - Output parameters"
  set g [frame $tl.par]
  pack $g -side top
  VarBox2 $g 1 kyrref   "Reference year for climate model output" ""
  VarBox2 $g 2 iy0      "First year for climate model output"     ""
  VarBox2 $g 3 lastyear "Last year for climate model run"         ""
  VarBox2 $g 4 itemprt  "Printout interval for climate model"     ""
  set f [frame $tl.buts]
  pack $f -side top
  button $f.ok -text OK -command "destroy $tl"
  button $f.help -text Help -command "Help outyear.htm"
  pack $f.ok $f.help -side left
}

proc SeaLevelParams {} {
  global TitleBanner
  set tl [toplevel .slparam]
  wm title $tl "$TitleBanner - Sea Level Model parameters"
#  VarBox $tl.a zi0in    "ZI0"
#  VarBox $tl.b tauin    "SGIC tau"
#  VarBox $tl.c dtstarin "DTSTAR (lower bound value)"
#  VarBox $tl.d dtendin  "DTEND (upper bound for DTSTAR)"
#  VarBox $tl.e betagin  "Greenland sensitivity"
#  VarBox $tl.f beta1in  "Antarctic sensitivity (1)"
#  VarBox $tl.g beta2in  "Antarctic sensitivity (2)"
#  VarBox $tl.h db3in    "Antarctic initial mass balance"
  set g [frame $tl.par]
  pack $g -side top
  VarBox2 $g 1 zi0in    "ZI0"                            ""
  VarBox2 $g 2 tauin    "SGIC tau"                       ""
  VarBox2 $g 3 dtstarin "DTSTAR (lower bound value)"     ""
  VarBox2 $g 4 dtendin  "DTEND (upper bound for DTSTAR)" ""
  VarBox2 $g 5 betagin  "Greenland sensitivity"          ""
  VarBox2 $g 6 beta1in  "Antarctic sensitivity (1)"      ""
  VarBox2 $g 7 beta2in  "Antarctic sensitivity (2)"      ""
  VarBox2 $g 8 db3in    "Antarctic initial mass balance" ""
  button $tl.ok -text OK -command "destroy $tl"
  pack $tl.ok -side top
  grab $tl
}

proc GCPdn80s {s} {
  if ("$s"=="normal") {set col "black"} else {set col "darkgray"}
  .gcparam.carbon.mode.l2    config -fg $col
  .gcparam.carbon.mode.dn80s config -state $s
}

proc GasCycleParams {} {
  global User Gascycle
  global Font
  global TitleBanner
  set tl [toplevel .gcparam]
  wm title $tl "$TitleBanner - Gas Cycle Models parameters"

  set f [frame $tl.carbon -relief sunken -bd 1]
  pack $f -side top -fill x -expand on
  label $f.title -text "Carbon Cycle Model"
  pack $f.title -side top

#  set g [frame $f.mode]
#  pack $g -side top -fill x -expand on
#  label $g.l1 -text "Model mode:"
#  radiobutton $g.low  -text "Low"  -variable Gascycle(ccmode) -value 1
#  radiobutton $g.best -text "Best" -variable Gascycle(ccmode) -value 2
#  radiobutton $g.high -text "High" -variable Gascycle(ccmode) -value 3
#  radiobutton $g.user -text "User" -variable Gascycle(ccmode) -value 4
#  label $g.l2 -text "Dn80s ="
#  entry $g.dn80s -textvariable User(duser) -width 10
#  pack $g.l1 $g.low $g.best $g.high $g.user $g.l2 $g.dn80s -side left

  set g [frame $f.dn80s]
  pack $g -side left -anchor w
  VarBox2 $g 1 duser "D!vn!080s =" ""

  set g [frame $f.methox]
  pack $g -side right
  label $g.l1 -text "Methane oxidation term:"
  checkbutton $g.chk -text "Include" -variable User(imeth)
  pack $g.l1 $g.chk -side left

  set f [frame $tl.methane -relief sunken -bd 1]
  pack $f -side top -fill x -expand on
  label $f.title -text "Methane Model"
  pack $f.title -side top
  set g [frame $f.life]
  pack $g -side top -fill x -expand on
#  label $g.l1 -text "CH4 Lifetime:"
  frame $g.l1
  CleverText $g.l1 "CH!v4!0 Lifetime:"
  radiobutton $g.low  -text "Low"        -variable User(itaumeth) -value 1
  radiobutton $g.best -text "Best Guess" -variable User(itaumeth) -value 2
  radiobutton $g.high -text "High"       -variable User(itaumeth) -value 3
  radiobutton $g.const -text "Constant"  -variable User(itaumeth) -value 4
#  text $g.t1 -font $Font(greek) -height 1 -width 1 -bd 0 -cursor left_ptr -takefocus 0
#  bindtags $g.t1 [list all]
#  $g.t1 insert end "t"
#  label $g.l2 -text " ="
  frame $g.t1
  CleverText $g.t1 "!gt!0 ="
  entry $g.tau -textvariable User(tau90ch4) -width 10
  pack $g.l1 $g.low $g.best $g.high $g.const $g.t1 $g.tau -side left

  set sbs [frame $tl.sidebyside]
  pack $sbs -side top -fill x -expand on

  set f [frame $sbs.n2o -relief sunken -bd 1]
  pack $f -side left -fill x -expand on
  label $f.title -text "Nitrous Oxide"
  pack $f.title -side top
  set g [frame $f.life]
  pack $g -side top -anchor w
  VarBox2 $g 1 taun2o "N!v2!0O Lifetime: !gt!0 =" ""

#  label $g.l1 -text "N2O Lifetime:"
#  checkbutton $g.chk -text "User" -variable Gascycle(n2olife)
#  text $g.t1 -font $Font(greek) -height 1 -width 1 -bd 0 -cursor left_ptr -takefocus 0
#  bindtags $g.t1 [list all]
#  $g.t1 insert end "t"
#  label $g.l2 -text " ="
#  entry $g.tau -textvariable User(taun2o) -width 10
#  pack $g.l1 $g.chk $g.t1 $g.l2 $g.tau -side left

  set f [frame $sbs.ozone -relief sunken -bd 1]
  pack $f -side left -fill both -expand on
  label $f.title -text "Stratospheric Ozone"
  pack $f.title -side top
  set g [frame $f.fb]
  pack $g -side top -fill x -expand on -anchor nw
  label $g.l1 -text "Feedback:"
  checkbutton $g.chk -text "Include" -variable User(io3feed)
  pack $g.l1 $g.chk -side left

  set f [frame $tl.aerosol -relief sunken -bd 1]
  pack $f -side top -fill x -expand on
  label $f.title -text "Aerosol Forcing"
  pack $f.title -side top
  set g [frame $f.grd]
  pack $g -side top -anchor w
  VarBox2 $g 1 s90dir "Direct:"     ""
  VarBox2 $g 2 s90ind "Indirect:"   ""
  VarBox2 $g 3 s90bio "Biospheric:" ""

  button $tl.ok -text OK -command "destroy $tl"
  pack $tl.ok -side top
  grab $tl
}


proc ClimateParams {} {
  global Default User
  global TitleBanner
  set tl [toplevel .cliparam]
  wm title $tl "$TitleBanner - Climate Model parameters"
  set f [frame $tl.par]
  pack $f -side top
  set g [label $f.e -text "Default"]
  grid $g -column 4 -row 0

  set deg "\xb0"
#  VarBox2 $f 1 dt2xuser  "Sensitivity (!gD!0T!v2x!0)"            "!^o!0C"
  VarBox2 $f 1 dt2xuser  "Sensitivity (!gD!0T!v2x!0)"            "${deg}C"
  VarBox2 $f 2 yk        "Diffusivity (K)"                       "cm!^2!0 / sec"
  VarBox2 $f 3 hm        "Mixed layer depth (h)"                 "m"
  VarBox2 $f 4 pi        "Sinking water !gD!0T factor (!gp!0)"   ""

  set g [label $f.a5 -text "Upwelling rate"]
  grid $g -column 0 -row 5 -sticky e
  set g [frame $f.b5]
  grid $g -column 1 -row 5 -columnspan 2
  radiobutton $g.c -text "constant" -variable User(ivarw) -value 0 -command {WT0state disabled}
  radiobutton $g.v -text "variable" -variable User(ivarw) -value 1 -command {WT0state normal  }
  pack $g.c $g.v -side left

  VarBox2 $f 6 w0    "Initial upwelling rate (W!v0!0)"           "m / yr"
  $f.e6 config -text "( 4 * K )" -command {set User(w0) [expr 4 * $User(yk)]}
#  VarBox2 $f 7 tw0nh "Temperature at which W becomes zero, NH"   "!^o!0C"
#  VarBox2 $f 8 tw0sh "Temperature at which W becomes zero, SH"   "!^o!0C"
  VarBox2 $f 7 tw0nh "Temperature at which W becomes zero, NH"   "${deg}C"
  VarBox2 $f 8 tw0sh "Temperature at which W becomes zero, SH"   "${deg}C"

  label $tl.spacer -text " " -height 1
  button $tl.ok -text OK -command "destroy $tl"
  pack $tl.spacer $tl.ok -side top
  grab $tl
}

proc WT0state {state} {
  set f .cliparam.par
  if {[string compare $state "normal"]==0} {
    set col black
  } else {
    set col #808080
  }
  $f.a7.l1   config -fg $col
  $f.b7      config -fg $col -state $state
  $f.c7.l2.a config -fg $col
  $f.c7.l3   config -fg $col
  $f.e7      config -state $state
  $f.a8.l1   config -fg $col
  $f.b8      config -fg $col -state $state
  $f.c8.l2.a config -fg $col
  $f.c8.l3   config -fg $col
  $f.e8      config -state $state
}

proc VarBox2 {f row var lbl1 lbl2} {
  global Default User

  set g [frame $f.a$row]
  grid $g -column 0 -row $row -sticky e
  CleverText $g $lbl1

  set g [entry $f.b$row -width 10 -textvariable User($var) -justify right]
  grid $g -column 1 -row $row

  set g [frame $f.c$row -height 0]
  grid $g -column 2 -row $row -sticky w
  CleverText $g $lbl2
  grid columnconfigure $f 2 -minsize 40

  set g [label $f.d$row -text " "]
  grid $g -column 3 -row $row

  set g [button $f.e$row -text $Default($var) -width 6 -justify right -command "set User($var) $Default($var)"]
  grid $g -column 4 -row $row -sticky ew
}

proc CleverText {f txt} {
  global Font
  label $f.dummy -text "" -bd 0
  pack $f.dummy -side left
  set l [split "!0$txt" !]
  set n [llength $l]
  set i 1
  while {$i<$n} {
    set s [lindex $l $i]
    set t [string range $s 1 end]
    switch -exact [string range $s 0 0] {
      0  { set greek 0; set super 0; set sub 0; set symbol 0 }
      g  { set greek  1 }
      \^ { set super  1 }
      v  { set sub    1 }
      s  { set symbol 1 }
    }
    if {[string length $t]>0} {
      set flags ""
      if {$greek} {
        label $f.l$i -text $t -font $Font(greek) -highlightthickness 0 -padx 0 -pady 0
      } else {
        if {$symbol} {
          label $f.l$i -text $t -font $Font(symbola) -highlightthickness 0 -padx 0 -pady 0
        } else {
          if {$super} {
            frame $f.l$i
            label $f.l$i.a -text $t  -bd 0 -highlightthickness 0 -padx 0 -pady 0
            label $f.l$i.b -text " " -bd 0 -highlightthickness 0 -padx 0 -pady 0
            pack $f.l$i.a $f.l$i.b -side top
          } else {
            if {$sub} {
              frame $f.l$i
              label $f.l$i.a -text $t  -bd 0 -highlightthickness 0 -padx 0 -pady 0
              label $f.l$i.b -text " " -bd 0 -highlightthickness 0 -padx 0 -pady 0
              pack $f.l$i.b $f.l$i.a -side top
            } else {
              label $f.l$i -text $t
            }
          }
        }
      }
      $f.l$i config -bd 0 -highlightthickness 0
      pack $f.l$i -side left
    }
    incr i
  }
}

proc RadioSelection {frame variable title number} {

    # Creates a list of radiobuttons for the chosen variable; if
    # $number is 2, the options are "On" and "Off", with value 1 and 0
    # respectively; if $number is 3, it's Hi/Mid/Lo, values 3/2/1.

    label $frame.title -text $title 
    pack $frame.title -side top -anchor w

    set f [frame $frame.radio]
    pack $f -side top -anchor w

    if {$number == 3} {
	radiobutton $f.high -text "High" -variable $variable -value 3 -highlightthickness 0
	radiobutton $f.mid  -text "Mid"  -variable $variable -value 2 -highlightthickness 0
	radiobutton $f.low  -text "Low"  -variable $variable -value 1 -highlightthickness 0
	pack $f.high $f.mid $f.low -side left
    }
    if {$number == 2} {
	radiobutton $f.on  -text "On"  -variable $variable -value 1 -highlightthickness 0
	radiobutton $f.off -text "Off" -variable $variable -value 0 -highlightthickness 0
	pack $f.on $f.off -side left
    }
    return $frame
}


proc UNEPparams {} {
# UNEP has a cut-down selection of parameters, which all fit on one dialog
  global User Gascycle Font RunMode TitleBanner

  if {[winfo exists .uneppar]} {
    focus .uneppar
    return
  }

  set tl [toplevel .uneppar]
  wm title $tl "$TitleBanner model parameters"

  set f [frame $tl.forcing]
  pack $f -side top -fill x -expand on
  label $f.title -text "Forcing Controls"
  pack $f.title -side top

  frame $f.ccm -relief sunken -bd 2
  RadioSelection $f.ccm User(ccm) "Carbon Cycle Model" 3

  frame $f.cccf -relief sunken -bd 2 
  RadioSelection $f.cccf User(cccf) "C-cycle Climate Feedbacks" 2

  frame $f.aerosol -relief sunken -bd 2 
  RadioSelection $f.aerosol User(aerosol) "Aerosol Forcing" 3

  pack $f.ccm $f.cccf $f.aerosol -side top -fill x

  set f [frame $tl.cmparams]
  pack $f -side top -fill x -expand on
  label $f.title -text "Climate Model Parameters"
  pack $f.title -side top
  
  frame $f.climsen  -relief sunken -bd 2 
  set deg "\xb0"
  frame $f.climsen.title
  frame $f.climsen.units
  CleverText $f.climsen.title "Sensitivity (!gD!0T!v2x!0)"
  entry $f.climsen.entry -width 8 -textvariable User(dt2xuser) -justify right
  CleverText $f.climsen.units "${deg}C"
  pack $f.climsen.title $f.climsen.entry $f.climsen.units -side left

  pack $f.climsen -side top -fill x

  frame $f.thc  -relief sunken -bd 2
  RadioSelection $f.thc User(thc) "Thermohaline Circulation" 2
  $f.thc.radio.on configure -text "Variable" -value 2
  $f.thc.radio.off configure -text "Constant" -value 0

  pack $f.thc -side top -fill x

  frame $f.vdiff -relief sunken -bd 2
  frame $f.vdiff.title
  frame $f.vdiff.units
  CleverText $f.vdiff.title "Vert. Diffus. (K!vz!0):"
  entry $f.vdiff.entry -width 5 -textvariable User(kzuser) -justify right
  CleverText $f.vdiff.units "cm!^2!0/s"
  pack $f.vdiff.title $f.vdiff.entry $f.vdiff.units -side left

  pack $f.vdiff -side top -fill x

  frame $f.ice  -relief sunken -bd 2
  RadioSelection $f.ice User(ice) "Ice Melt" 3

  pack $f.ice -side top -fill x

  set f [frame $f.model  -relief sunken -bd 2]
  pack $f -side top -fill x
  label $f.title -text "Model:"
  menubutton $f.model -textvariable User(modelname) -menu $f.model.menu -bg white
  set models {User GFDL CSIRO HadCM3 HadCM2 ECH4/OPYC CSM PCM}
  set mu [menu $f.model.menu -tearoff no]
  for {set i 0} {$i <=7} {incr i} {
      set m [lindex $models $i]
      $mu add radio -variable User(model) -value $i -label $m \
	      -command "set User(modelname) $m"
  }
  pack $f.title $f.model -side left -anchor w


#  set f [frame $tl.carbon -relief sunken -bd 1]
#  pack $f -side top -fill x -expand on
#  label $f.title -text "Carbon Cycle Model"
#  pack $f.title -side top
#  set g [frame $f.dn80s]
#  pack $g -side top -anchor e
#  VarBox2 $g 1 duser "D!vn!080s =" "GtC"
#
#  set f [frame $tl.aerosol -relief sunken -bd 1]
#  pack $f -side top -fill x -expand on
#  label $f.title -text "Current Aerosol Forcing"
#  pack $f.title -side top
#  set g [frame $f.grd]
#  pack $g -side top -anchor e
#  VarBox2 $g 1 s90dir "Direct:"     "Wm!^-2!0"
#  VarBox2 $g 2 s90ind "Indirect:"   "Wm!^-2!0"
#  # UNDP has hard-wired biospheric
#  if {$RunMode=="UNEP"} {
#    VarBox2 $g 3 s90bio "Biospheric:" "Wm!^-2!0"
#  }
#
#  set f [frame $tl.climate -relief sunken -bd 1]
#  pack $f -side top -fill x -expand on
#  label $f.title -text "Climate Model"
#  pack $f.title -side top
#  set g [frame $f.grd]
#  pack $g -side top -anchor e
#  set deg "\xb0"
#  VarBox2 $g 1 dt2xuser  "Sensitivity (!gD!0T!v2x!0)"  "${deg}C"
#
#  # UNDP has hard-wired upwelling
#  if {$RunMode=="UNEP"} {
#    set g [frame $f.upwel]
#    pack $g -side top -anchor e
#    label $g.lbl -text "Upwelling rate"
#    radiobutton $g.c -text "constant" -variable User(ivarw) -value 0
#    radiobutton $g.v -text "variable" -variable User(ivarw) -value 1
#    pack $g.lbl $g.c $g.v -side left
#  }

  set f [frame $tl.buts]
  pack $f -side top -fill x
  button $f.ok -text OK -command "destroy $tl"
  button $f.help -text Help -command "Help modpar.htm"
  pack $f.ok $f.help -side left -expand on
}



