# files.tcl - file handling for MAGICC

proc ReadConcs { arrname filename } {
  upvar $arrname arr
  set fid [open $filename r]
  gets $fid line
  set n 0
  set done 0
  while {!$done} {
    gets $fid line
    if {[string range $line 1 4]=="YEAR"} { set done 1 } else {
      set arr(Year,$n)    [string trim [string range $line  1  4]]
      set arr(CO2user,$n) [string trim [string range $line  6 12]]
      set arr(CO2lo,$n)   [string trim [string range $line 14 20]]
      set arr(CO2mid,$n)  [string trim [string range $line 22 28]]
      set arr(CO2hi,$n)   [string trim [string range $line 30 36]]
      set arr(CH4user,$n) [string trim [string range $line 38 44]]
      set arr(CH4lo,$n)   [string trim [string range $line 46 52]]
      set arr(CH4mid,$n)  [string trim [string range $line 54 60]]
      set arr(CH4hi,$n)   [string trim [string range $line 62 68]]
      set arr(N2O,$n)     [string trim [string range $line 70 76]]
      incr n
    }
  }
  set arr(N) $n
  close $fid
}

proc UniqueName {} {
  global UniqueNameState
  if {![info exists UniqueNameState]} {set UniqueNameState 0}
  incr UniqueNameState
  return uniq$UniqueNameState
}

proc ViewFile {filename title} {
  set name .[UniqueName]
  toplevel $name
  wm title $name $title
  set f [frame $name.buts]
  pack $f -side bottom
  button $f.okay -text "OK" -command "destroy $name"
  button $f.print -text "Print" -command "PrintFile $filename"
  button $f.help -text "Help" -command "Help report.htm"
  pack $f.okay $f.print $f.help -side left
  set t [text $name.txt -setgrid true -wrap none -width 75 -height 25 -yscrollcommand "$name.sy set" -xscrollcommand "$name.b.sx set"]
  scrollbar $name.sy -orient vert -command "$t yview" -width 10
  frame $name.b
  scrollbar $name.b.sx -orient horiz -command "$t xview" -width 10
  set pad [expr [$name.sy cget -width]+2*([$name.sy cget -bd]+[$name.sy cget -highlightthickness])]
  frame $name.b.pad -width $pad
  pack $name.b -side bottom -fill x
  pack $name.b.pad -side right
  pack $name.b.sx -side bottom -fill x
  pack $name.sy -side right -fill y
  pack $name.txt -side right -fill both -expand true

  set fid [open $filename r]
  while {[gets $fid line]>=0} {
    $name.txt insert end "$line\n"
  }
  close $fid
}

proc PrintFile {filename} {
  global tcl_platform
  set txt ""
  set fid [open $filename r]
  while {[gets $fid line]>=0} {
    append txt "$line\n"
  }
  close $fid
  switch $tcl_platform(platform) {
    windows {
      PrintString $txt
    }
    unix {
      exec print3 $filename
    }
  }
}

proc PrintString {txt} {
  global tcl_platform
  switch $tcl_platform(platform) {
    windows {
      wprint_text $txt 100 15 0 0 0
    }
  }
}
