# graphs.tcl - MAGICC results graphs


proc SetGraphGeometry {} {
  global Graph
  set Graph(left)    30
  set Graph(right)  370
  set Graph(top)     60
  set Graph(bottom) 440
  set Graph(canwid) 520
  set Graph(canhgt) 450
}

# ---------------------- EMISSIONS -----------------------

proc EmisGraphs {} {
  global EmisGas Graph File
  global TitleBanner

  if {[winfo exists .emis]} {
    focus .emis
    return
  }

  set EmisGas CO2
  toplevel .emis
  wm title .emis "$TitleBanner - Gas Emissions"
  set f [frame .emis.buts]
  pack $f -side left -fill y
  set g [frame $f.gas -relief sunken -bd 2]
  pack $g -side top -fill x
  radiobutton $g.co2 -variable EmisGas -value CO2 -text "CO2" -command {EmisUpd}
  radiobutton $g.ch4 -variable EmisGas -value CH4 -text "CH4" -command {EmisUpd}
  radiobutton $g.n2o -variable EmisGas -value N2O -text "N2O" -command {EmisUpd}
  radiobutton $g.so2 -variable EmisGas -value SO2 -text "SO2" -command {EmisUpd}
  pack $g.co2 $g.ch4 $g.n2o $g.so2 -side top -anchor w

  BottomButtons $f .emis gremis.htm

  SetGraphGeometry
  canvas .emis.graph -width $Graph(canwid) -height $Graph(canhgt)
  pack .emis.graph -side right

  Mag_Init
  Mag_ReadDIS emiss.dp 1 9 pCO2foss pCO2defo pCH4 pN2O pSO21 pSO22 pSO23 pSO2g
  Mag_ReadDIS emiss.dr 1 9 rCO2foss rCO2defo rCH4 rN2O rSO21 rSO22 rSO23 rSO2g

  # disaggregate SO2 1990 total into regions
  # subtract 75 from each; add 37, 28,10 to NAM-EUR, ASIA, RoW
#  Mag_AlterSeries pSO21 1.0 -38.0
#  Mag_AlterSeries pSO22 1.0 -47.0
#  Mag_AlterSeries pSO23 1.0 -65.0
#  Mag_AlterSeries rSO21 1.0 -38.0
#  Mag_AlterSeries rSO22 1.0 -47.0
#  Mag_AlterSeries rSO23 1.0 -65.0

  set Graph(canvas) .emis.graph
  Mag_GraphCanvas .emis.graph $Graph(left) $Graph(top) $Graph(right) $Graph(bottom)
  KeyPosition [expr $Graph(right)+10] [expr $Graph(top)+10]

  set Graph(pol) [GetProfileDesc $File(policy)]
  set Graph(ref) [GetProfileDesc $File(reference)]

  set Graph(initXrange) 1
  EmisUpd
}

proc EmisUpd {} {
  global EmisGas
  .emis.graph delete all
  update
  set green  #00c000
  set yellow #ffff00
  set purple #ff00ff
  set blue   #0000ff
  set orange #e08000
  set red    #ff4040
  set cyan   #00ffff
  set pink   #ff0080
  switch $EmisGas {
    CO2 {
      SetLimits pCO2foss pCO2defo rCO2foss rCO2defo
#      TimeRangeFixed
      DrawGraticule
      Mag_DrawCurve  pCO2defo        $yellow 
      Mag_DrawCurve  rCO2defo        $blue   
      Mag_DrawCurve  pCO2foss        $green
      Mag_DrawCurve  rCO2foss        $purple
      DrawAxes
      MainTitle "Carbon Dioxide Emissions (Gt C)"
      ProfileTitles
      KeyClear
      KeyAdd rfoss  "Reference fossil fuel"   $purple solid  1
      KeyAdd rdefo  "Reference deforest."     $blue   solid  1
      KeyGap gap1
      KeyAdd pfoss  "Policy fossil fuel"      $green  solid  1
      KeyAdd pdefo  "Policy deforestation"    $yellow solid  1
      KeyDisplay
      SwitchVisInit .emis.buts
      SwitchVisSet rCO2foss  rfoss  "Ref. foss."
      SwitchVisSet rCO2defo  rdefo  "Ref. defo."
      SwitchVisSet pCO2foss  pfoss  "Pol. foss."
      SwitchVisSet pCO2defo  pdefo  "Pol. defo."
    }
    CH4 {
      SetLimits pCH4 rCH4
#      TimeRangeFixed
      DrawGraticule
      Mag_DrawCurve  pCH4        $green
      Mag_DrawCurve  rCH4        $purple
      DrawAxes
      MainTitle "Methane Emissions (Tg CH4)"
      ProfileTitles
      KeyClear
      KeyAdd ref  "Reference"   $purple solid  1
      KeyGap gap1
      KeyAdd pol  "Policy"      $green  solid  1
      KeyDisplay
      SwitchVisInit .emis.buts
      SwitchVisSet rCH4 ref "Ref."
      SwitchVisSet pCH4 pol "Pol."
    }
    N2O {
      SetLimits pN2O rN2O
#      TimeRangeFixed
      DrawGraticule
      Mag_DrawCurve  pN2O        $green
      Mag_DrawCurve  rN2O        $purple
      DrawAxes
      MainTitle "Nitrous Oxide Emissions (Tg N)"
      ProfileTitles
      KeyClear
      KeyAdd ref  "Reference"   $purple solid  1
      KeyGap gap1
      KeyAdd pol  "Policy"      $green  solid  1
      KeyDisplay
      SwitchVisInit .emis.buts
      SwitchVisSet rN2O ref "Ref."
      SwitchVisSet pN2O pol "Pol."
    }
    SO2 {
      SetLimits pSO21 pSO22 pSO23 pSO2g rSO21 rSO22 rSO23 rSO2g
#      TimeRangeFixed
      DrawGraticule
      Mag_DrawCurve  pSO2g        $green
      Mag_DrawCurve  rSO2g        $purple
      Mag_DrawCurve  pSO21        $cyan
      Mag_DrawCurve  rSO21        $pink
      Mag_DrawCurve  pSO22        $yellow
      Mag_DrawCurve  rSO22        $blue
      Mag_DrawCurve  pSO23        $red
      Mag_DrawCurve  rSO23        $orange
      DrawAxes
      MainTitle "Sulphur Dioxide Emissions (Tg S)"
      ProfileTitles
      KeyClear
      KeyAdd refg  "Reference Global"    $purple solid  1
      KeyAdd ref1  "Reference NAM-EUR"   $pink   solid  1
      KeyAdd ref2  "Reference ASIA"      $blue   solid  1
      KeyAdd ref3  "Reference RoW"       $orange solid  1
      KeyGap gap1
      KeyAdd polg  "Policy Global"       $green  solid  1
      KeyAdd pol1  "Policy NAM-EUR"      $cyan   solid  1
      KeyAdd pol2  "Policy ASIA"         $yellow solid  1
      KeyAdd pol3  "Policy RoW"          $red    solid  1
      KeyDisplay
      SwitchVisInit .emis.buts
      SwitchVisSet rSO2g refg "Ref. Global"
      SwitchVisSet pSO2g polg "Pol. Global"
      SwitchVisSet rSO21 ref1 "Ref. NAM-EUR"
      SwitchVisSet pSO21 pol1 "Pol. NAM-EUR"
      SwitchVisSet rSO22 ref2 "Ref. ASIA"
      SwitchVisSet pSO22 pol2 "Pol. ASIA"
      SwitchVisSet rSO23 ref3 "Ref. RoW"
      SwitchVisSet pSO23 pol3 "Pol. RoW"
    }

  }
}

# -------------------- CONCENTRATIONS ---------------------

proc ConcGraphs {} {
  global ConcGas Graph File
  global TitleBanner

  if {[winfo exists .concs]} {
    focus .concs
    return
  }
  set ConcGas CO2
  toplevel .concs
  wm title .concs "$TitleBanner - Gas Concentrations"
  set f [frame .concs.buts]
  pack $f -side left -fill y
  set g [frame $f.gas -relief sunken -bd 2]
  pack $g -side top -fill x
  radiobutton $g.co2 -variable ConcGas -value CO2 -text "CO2" -command {ConcUpd}
  radiobutton $g.ch4 -variable ConcGas -value CH4 -text "CH4" -command {ConcUpd}
  radiobutton $g.n2o -variable ConcGas -value N2O -text "N2O" -command {ConcUpd}
  pack $g.co2 $g.ch4 $g.n2o -side top -anchor w

  BottomButtons $f .concs grconc.htm

  SetGraphGeometry
  canvas .concs.graph -width $Graph(canwid) -height $Graph(canhgt)
  pack .concs.graph -side right

  Mag_Init
  Mag_ReadDIS concs.dp 1 8 dpCO2user dpCO2lo dpCO2mid dpCO2hi dpCH4user dpCH4lo dpCH4mid dpCH4hi dpN2O
  Mag_ReadDIS concs.dr 1 8 drCO2user drCO2lo drCO2mid drCO2hi drCH4user drCH4lo drCH4mid drCH4hi drN2O
  Mag_ReadDIS concs.up 1 8 upCO2user upCO2lo upCO2mid upCO2hi upCH4user upCH4lo upCH4mid upCH4hi upN2O
  Mag_ReadDIS concs.ur 1 8 urCO2user urCO2lo urCO2mid urCO2hi urCH4user urCH4lo urCH4mid urCH4hi urN2O
  set Graph(canvas) .concs.graph
  Mag_GraphCanvas .concs.graph $Graph(left) $Graph(top) $Graph(right) $Graph(bottom)
  KeyPosition [expr $Graph(right)+10] [expr $Graph(top)+10]

  set Graph(pol) [GetProfileDesc $File(policy)]
  set Graph(ref) [GetProfileDesc $File(reference)]

  set Graph(initXrange) 1
  ConcUpd
}

proc ConcUpd {} {
  global ConcGas Switch
  .concs.graph delete all
  update
  set green  #00c000
  set yellow #ffff00
  set purple #ff00ff
  set blue   #0000ff
  switch $ConcGas {
    CO2 {
      SetLimits urCO2user urCO2lo urCO2mid urCO2hi upCO2user upCO2lo upCO2mid upCO2hi
      DrawGraticule
      Mag_DrawRegion2 upCO2lo upCO2hi $green  2 4
      Mag_DrawRegion2 urCO2lo urCO2hi $purple 1 4
      Mag_DrawCurve  upCO2user       $yellow
      Mag_DrawCurve  urCO2user       $blue
      Mag_DrawCurve  dpCO2user       $green
      Mag_DrawCurve  drCO2user       $purple
      DrawAxes
      MainTitle "Carbon Dioxide Concentration (ppmv)"
      ProfileTitles
      KeyClear
      KeyAdd urhilo "Reference Range"      $purple range2 1
      KeyAdd urmid  "Reference Best Guess" $purple solid  1
      KeyAdd uruser "Reference User Model" $blue   solid  1
      KeyGap gap1
      KeyAdd uphilo "Policy Range"         $green  range1 1
      KeyAdd upmid  "Policy Best Guess"    $green  solid  1
      KeyAdd upuser "Policy User Model"    $yellow solid  1
      KeyDisplay
      SwitchVisInit .concs.buts
      SwitchVisSet urCO2lo   urhilo "Ref. range"
      SwitchVisSet drCO2user  urmid  "Ref. best"
#	SwitchVisSet urCO2mid  urmid  "Ref. best"
      SwitchVisSet urCO2user uruser "Ref. user"
      SwitchVisSet upCO2lo   uphilo "Pol. range"
#      SwitchVisSet upCO2mid  upmid  "Pol. best"
      SwitchVisSet dpCO2user  upmid  "Pol. best"
      SwitchVisSet upCO2user upuser "Pol. user"
      TimeRangeFrame .concs.buts ConcUpd
      $Switch(frame).urhilo invoke
      $Switch(frame).uruser invoke
      $Switch(frame).uphilo invoke
#      $Switch(frame).upmid invoke
      $Switch(frame).upuser invoke
    }
    CH4 {
      SetLimits urCH4user urCH4lo urCH4mid urCH4hi upCH4user upCH4lo upCH4mid upCH4hi
      DrawGraticule
      Mag_DrawRegion2 upCH4lo upCH4hi $green   2 4
      Mag_DrawRegion2 urCH4lo urCH4hi $purple  1 4
      Mag_DrawCurve  upCH4user       $yellow
      Mag_DrawCurve  urCH4user       $blue
      Mag_DrawCurve  upCH4mid        $green
      Mag_DrawCurve  urCH4mid        $purple
      DrawAxes
      MainTitle "Methane Concentration (ppbv)"
      ProfileTitles
      KeyClear
      KeyAdd urhilo "Reference Range"      $purple range2 1
      KeyAdd urmid  "Reference Best Guess" $purple solid  1
      KeyAdd uruser "Reference User Model" $blue   solid  1
      KeyGap gap1
      KeyAdd uphilo "Policy Range"         $green  range1 1
      KeyAdd upmid  "Policy Best Guess"    $green  solid  1
      KeyAdd upuser "Policy User Model"    $yellow solid  1
      KeyDisplay
      SwitchVisInit .concs.buts
      SwitchVisSet urCH4lo   urhilo "Ref. range"
      SwitchVisSet urCH4mid  urmid  "Ref. best"
      SwitchVisSet urCH4user uruser "Ref. user"
      SwitchVisSet upCH4lo   uphilo "Pol. range"
      SwitchVisSet upCH4mid  upmid  "Pol. best"
      SwitchVisSet upCH4user upuser "Pol. user"
      TimeRangeFrame .concs.buts ConcUpd
      $Switch(frame).urhilo invoke
      $Switch(frame).uruser invoke
      $Switch(frame).uphilo invoke
#      $Switch(frame).upmid invoke
      $Switch(frame).upuser invoke
    }
    N2O {
      SetLimits dpN2O drN2O urN2O upN2O
      DrawGraticule
      Mag_DrawCurve  urN2O  $blue
      Mag_DrawCurve  dpN2O  $green
      Mag_DrawCurve  upN2O  $yellow
      Mag_DrawCurve  drN2O  $purple
      DrawAxes
      MainTitle "Nitrous Oxide Concentration (ppbv)"
      ProfileTitles
      KeyClear
      KeyAdd refmid  "Reference Best Guess" $purple solid 1
      KeyAdd refuser "Reference User"       $blue   solid 1
      KeyGap gap1
      KeyAdd polmid  "Policy Best Guess"    $green  solid 1
      KeyAdd poluser "Policy User"          $yellow solid 1
      KeyDisplay
      SwitchVisInit .concs.buts
      SwitchVisSet drN2O  refmid   "Ref. best"
      SwitchVisSet urN2O  refuser  "Ref. user"
      SwitchVisSet dpN2O  polmid   "Pol. best"
      SwitchVisSet upN2O  poluser  "Pol. user"
      TimeRangeFrame .concs.buts ConcUpd
      $Switch(frame).refuser invoke
#      $Switch(frame).polmid invoke
      $Switch(frame).poluser invoke
    }

  }
}

# ---------------------- RADIATIVE FORCING -----------------------

proc ForcGraphs {} {
  global Graph File
  global TitleBanner

  if {[winfo exists .forc]} {
    focus .forc
    return
  }
  toplevel .forc
  wm title .forc "$TitleBanner - Radiative Forcing"
  set f [frame .forc.buts]
  pack $f -side left -fill y

  Mag_Init
  Mag_ReadDIS forcings.dp 1 8 dpCO2 dpCH4 dpN2O dpHalos dpTropO3 dpSO4dir dpSO4ind dpBioAer dpTotal
  Mag_ReadDIS forcings.dr 1 8 drCO2 drCH4 drN2O drHalos drTropO3 drSO4dir drSO4ind drBioAer drTotal
  Mag_ReadDIS forcings.up 1 8 upCO2 upCH4 upN2O upHalos upTropO3 upSO4dir upSO4ind upBioAer upTotal
  Mag_ReadDIS forcings.ur 1 8 urCO2 urCH4 urN2O urHalos urTropO3 urSO4dir urSO4ind urBioAer urTotal

  SetGraphGeometry
  canvas .forc.graph -width $Graph(canwid) -height $Graph(canhgt)
  pack .forc.graph -side right
  set Graph(canvas) .forc.graph
  Mag_GraphCanvas .forc.graph $Graph(left) $Graph(top) $Graph(right) $Graph(bottom)
  
  set blank [image create bitmap -data {
    #define blank_width 6
    #define blank_height 6
    static char blank_bits[] = {
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
  }]
    
  set g [frame $f.matrix -relief sunken -bd 2]
  pack $g -side top -fill x
  set x 0
  foreach file { Def/Usr\nRef/Pol d\nr d\np u\nr u\np } {
    label $g.lbl$file -text $file
    grid $g.lbl$file -row 0 -column $x
    incr x
  }
  set y 1
  foreach gas { CO2 CH4 N2O Halos TropO3 SO4dir SO4ind BioAer Total } {
    label $g.lbl$gas -text $gas
    grid $g.lbl$gas -row $y -column 0 -sticky e
    set x 1
    foreach file { dr dp ur up } {
      set Graph(forc.$file$gas) 0
      checkbutton $g.$file$gas -variable Graph(forc.$file$gas) -image $blank -indicatoron false -command "ForcSwitch $file$gas"
      grid $g.$file$gas -row $y -column $x
      incr x
    }
    incr y
  }
  
  button $f.alloff -text "All off" -command "ForcSwAll 0"
  button $f.allon  -text "All on"  -command "ForcSwAll 1"
  pack $f.alloff $f.allon -side top -fill x

  BottomButtons $f .forc grforc.htm
 
  KeyPosition [expr $Graph(right)+10] [expr $Graph(top)+10]

  set Graph(pol) [GetProfileDesc $File(policy)]
  set Graph(ref) [GetProfileDesc $File(reference)]

  set Graph(initXrange) 1
  ForcUpd
}

proc ForcSwitch { curve } {
  global Graph

  if {$Graph(forc.$curve)} {
    $Graph(canvas) move $curve 0 -1000
  } else {
    $Graph(canvas) move $curve 0 1000
  }
}


proc ForcSwAll { state } {
  global Graph

  foreach gas { CO2 CH4 N2O Halos TropO3 SO4dir SO4ind BioAer Total } {
    foreach file { dr dp ur up } {
      if {$Graph(forc.$file$gas)!=$state} {
        .forc.buts.matrix.$file$gas invoke
      }
    }
  }
}


proc ForcUpd {} {
  .forc.graph delete all
  update
  set green  #00c000
  set yellow #ffff00
  set purple #ff00ff
  set blue   #0000ff
  set black  #000000

  SetLimits \
    dpCO2 dpCH4 dpN2O dpHalos dpTropO3 dpSO4dir dpSO4ind dpBioAer dpTotal \
    drCO2 drCH4 drN2O drHalos drTropO3 drSO4dir drSO4ind drBioAer drTotal \
    upCO2 upCH4 upN2O upHalos upTropO3 upSO4dir upSO4ind upBioAer upTotal \
    urCO2 urCH4 urN2O urHalos urTropO3 urSO4dir urSO4ind urBioAer urTotal
#  TimeRangeFixed

  DrawGraticule
  set colour "$green $yellow $purple $blue"
  set symbol "1 2 3 4 5 6 7 8 0"
  set y 1
  foreach gas { CO2 CH4 N2O Halos TropO3 SO4dir SO4ind BioAer Total } {
    set x 1
    foreach file { dr dp ur up } {
      Mag_DrawCurve $file$gas [lindex $colour [expr $x-1]] [lindex $symbol [expr $y-1]] 10
      ForcSwitch $file$gas
      incr x
    }
    incr y
  }

  DrawAxes
  MainTitle "Radiative Forcing (W/m\xb2)"
  ProfileTitles
  KeyClear
  KeyAdd dr  "Default/Reference"   $green  solid 1
  KeyAdd dp  "Default/Policy"      $yellow solid 1
  KeyAdd ur  "User/Reference"      $purple solid 1
  KeyAdd up  "User/Policy"         $blue   solid 1
  KeyGap gap1
  KeyAdd co2 "CO2"                 $black  solid 1 1
  KeyAdd ch4 "CH4"                 $black  solid 1 2
  KeyAdd n2o "N2O"                 $black  solid 1 3
  KeyAdd hal "Halocarbons"         $black  solid 1 4
  KeyAdd to3 "Trop. Ozone"         $black  solid 1 5
  KeyAdd s4d "SO4 direct"          $black  solid 1 6
  KeyAdd s4i "SO4 indirect"        $black  solid 1 7
  KeyAdd bio "Biospheric Aerosols" $black  solid 1 8
  KeyAdd tot "Total"               $black  solid 1
  KeyDisplay

  #TimeRangeFrame .forc.buts ForcUpd
}


# ------------------ TEMPERATURE & SEA LEVEL ------------------

proc TslGraphs {} {
  global TempSea Graph File
  global TitleBanner

  if {[winfo exists .tsl]} {
    focus .tsl
    return
  }
  set TempSea temp
  toplevel .tsl
  wm title .tsl "$TitleBanner - Temperature & Sea Level"
  set f [frame .tsl.buts]
  pack $f -side left -fill y
  set g [frame $f.var -relief sunken -bd 2]
  pack $g -side top -fill x
  radiobutton $g.temp -variable TempSea -value temp -text "Temp" -command {TslUpd}
  radiobutton $g.sea  -variable TempSea -value sea  -text "Sea level"   -command {TslUpd}
  pack $g.temp $g.sea -side top -anchor w

  BottomButtons $f .tsl grtsl.htm

  SetGraphGeometry
  canvas .tsl.graph -width $Graph(canwid) -height $Graph(canhgt)
  pack .tsl.graph -side right

  Mag_Init
  Mag_ReadDIS temps.dp  1 9 dpTuser dpTlo dpTmid dpThi dpTnoso2
  Mag_ReadDIS temps.dr  1 9 drTuser drTlo drTmid drThi drTnoso2
  Mag_ReadDIS temps.up  1 9 upTuser upTlo upTmid upThi upTnoso2
  Mag_ReadDIS temps.ur  1 9 urTuser urTlo urTmid urThi urTnoso2
  Mag_ReadDIS sealev.dp 1 9 dpSuser dpSlo dpSmid dpShi dpSnoso2
  Mag_ReadDIS sealev.dr 1 9 drSuser drSlo drSmid drShi drSnoso2
  Mag_ReadDIS sealev.up 1 9 upSuser upSlo upSmid upShi upSnoso2
  Mag_ReadDIS sealev.ur 1 9 urSuser urSlo urSmid urShi urSnoso2

  set Graph(canvas) .tsl.graph
  Mag_GraphCanvas .tsl.graph $Graph(left) $Graph(top) $Graph(right) $Graph(bottom)
  KeyPosition [expr $Graph(right)+10] [expr $Graph(top)+10]

  set Graph(pol) [GetProfileDesc $File(policy)]
  set Graph(ref) [GetProfileDesc $File(reference)]

  set Graph(initXrange) 1
  TslUpd
}

proc TslUpd {} {
  global TempSea User Switch
  .tsl.graph delete all
  update
  set green  #00c000
  set yellow #ffff00
  set purple #ff00ff
  set blue   #0000ff
  set orange #e08000
  set red    #ff4040

  switch $TempSea {
    temp {
      SetLimits urTuser drTlo drTmid drThi drTnoso2 upTuser upTlo upTmid upThi upTnoso2
      DrawGraticule
      Mag_DrawRegion2 drTlo drThi $purple  2 4
      Mag_DrawRegion2 dpTlo dpThi $green   1 4
#      Mag_DrawCurve  drTnoso2      $orange
#      Mag_DrawCurve  dpTnoso2      $red
      Mag_DrawCurve  urTuser       $blue
      Mag_DrawCurve  upTuser       $yellow
      Mag_DrawCurve  drTuser       $purple
      Mag_DrawCurve  dpTuser       $green
      DrawAxes
      set deg "\xb0"
      MainTitle "Temperature Change (${deg}C) w.r.t. $User(kyrref)"
      ProfileTitles
      KeyClear
      KeyAdd drhilo  "Reference Range"      $purple range2 1
      KeyAdd drmid   "Reference Best Guess" $purple solid  1
      KeyAdd uruser  "Reference User Model" $blue   solid  1
#      KeyAdd drnoso2 "Reference con SO2"     $orange solid  1
      KeyGap gap1
      KeyAdd dphilo  "Policy Range"         $green  range1 1
      KeyAdd dpmid   "Policy Best Guess"    $green  solid  1
      KeyAdd upuser  "Policy User Model"    $yellow solid  1
#      KeyAdd dpnoso2 "Policy con SO2"        $red    solid  1
      KeyDisplay
      SwitchVisInit .tsl.buts
      SwitchVisSet drTlo    drhilo  "Ref. range"
#      SwitchVisSet drTmid   drmid   "Ref. best"
      SwitchVisSet drTuser   drmid   "Ref. best"
      SwitchVisSet urTuser  uruser  "Ref. user"
#      SwitchVisSet drTnoso2 drnoso2 "Ref. con SO2"
      SwitchVisSet dpTlo    dphilo  "Pol. range"
#      SwitchVisSet dpTmid   dpmid   "Pol. best"
      SwitchVisSet dpTuser   dpmid   "Pol. best"
      SwitchVisSet upTuser  upuser  "Pol. user"
#      SwitchVisSet dpTnoso2 dpnoso2 "Pol. con SO2"
      TimeRangeFrame .tsl.buts TslUpd
      $Switch(frame).drhilo invoke
      $Switch(frame).uruser invoke
#      $Switch(frame).drnoso2 invoke
      $Switch(frame).dphilo invoke
#      $Switch(frame).dpmid invoke
      $Switch(frame).upuser invoke
#      $Switch(frame).dpnoso2 invoke
    }
    sea {
      SetLimits urSuser drSlo drSmid drShi drSnoso2 upSuser upSlo upSmid upShi upSnoso2
      DrawGraticule
      Mag_DrawRegion2 drSlo drShi $purple  2 4
      Mag_DrawRegion2 dpSlo dpShi $green   1 4
 #     Mag_DrawCurve  drSnoso2      $orange
 #     Mag_DrawCurve  dpSnoso2      $red
      Mag_DrawCurve  urSuser       $blue
      Mag_DrawCurve  upSuser       $yellow
      Mag_DrawCurve  drSuser       $purple
      Mag_DrawCurve  dpSuser       $green
      DrawAxes
      MainTitle "Sea Level Change (cm) w.r.t. $User(kyrref)"
      ProfileTitles
      KeyClear
      KeyAdd drhilo  "Reference Range"      $purple range2 1
      KeyAdd drmid   "Reference Best Guess" $purple solid  1
      KeyAdd uruser  "Reference User Model" $blue   solid  1
  #    KeyAdd drnoso2 "Reference con SO2"     $orange solid  1
      KeyGap gap1
      KeyAdd dphilo  "Policy Range"         $green  range1 1
      KeyAdd dpmid   "Policy Best Guess"    $green  solid  1
      KeyAdd upuser  "Policy User Model"    $yellow solid  1
  #    KeyAdd dpnoso2 "Policy con SO2"        $red    solid  1
      KeyDisplay
      SwitchVisInit .tsl.buts
      SwitchVisSet drSlo    drhilo  "Ref. range"
 #     SwitchVisSet drSmid   drmid   "Ref. best"
      SwitchVisSet drSuser   drmid   "Ref. best"
      SwitchVisSet urSuser  uruser  "Ref. user"
  #    SwitchVisSet drSnoso2 drnoso2 "Ref. con SO2"
      SwitchVisSet dpSlo    dphilo  "Pol. range"
#      SwitchVisSet dpSmid   dpmid   "Pol. best"
      SwitchVisSet dpSuser   dpmid   "Pol. best"
      SwitchVisSet upSuser  upuser  "Pol. user"
  #    SwitchVisSet dpSnoso2 dpnoso2 "Pol. con SO2"
      TimeRangeFrame .tsl.buts TslUpd
      $Switch(frame).drhilo invoke
      $Switch(frame).uruser invoke
  #    $Switch(frame).drnoso2 invoke
      $Switch(frame).dphilo invoke
  #    $Switch(frame).dpmid invoke
      $Switch(frame).upuser invoke
  #    $Switch(frame).dpnoso2 invoke
    }
  }
}


# --------------------- COMMON PROCEDURES ---------------------

proc BottomButtons { f dlg {help {}} } {
  global RunMode

  button $f.okay -text "OK" -command "destroy $dlg" -width 11
  pack $f.okay -side bottom -fill x

  button $f.print -text "Print" -command "PrintGraph $dlg.graph"
  pack $f.print -side bottom -fill x

  button $f.eps -text "EPS file" -command "EPSfile $dlg.graph"
  pack $f.eps -side bottom -fill x

#button $f.ci -text "Caninsp" -command "caninsp $dlg.graph"
#pack $f.ci -side bottom -fill x

  if {$help!={}} {
    button $f.help -text "Help" -command "Help $help"
    pack $f.help -side bottom -fill x
  }
}

proc ChooseIntervals {} {
  global Graph
  set xr [expr $Graph(xmax)-$Graph(xmin)]
                 set Graph(xtick) 50 ; set Graph(xlabel) 100
  if {$xr<500} { set Graph(xtick) 25 ; set Graph(xlabel)  50 }
  if {$xr<250} { set Graph(xtick) 10 ; set Graph(xlabel)  25 }
  if {$xr<100} { set Graph(xtick)  5 ; set Graph(xlabel)  10 }

  set yr [expr $Graph(ymax)-$Graph(ymin)]
                  set Graph(ytick) 500 ; set Graph(ylabel) 1000
  if {$yr<5000} { set Graph(ytick) 250 ; set Graph(ylabel)  500 }
  if {$yr<2000} { set Graph(ytick) 100 ; set Graph(ylabel)  200 }
  if {$yr<1000} { set Graph(ytick)  50 ; set Graph(ylabel)  100 }
  if {$yr< 500} { set Graph(ytick)  25 ; set Graph(ylabel)   50 }
  if {$yr< 200} { set Graph(ytick)  10 ; set Graph(ylabel)   20 }
  if {$yr< 100} { set Graph(ytick)   5 ; set Graph(ylabel)   10 }
  if {$yr<  20} { set Graph(ytick)   1 ; set Graph(ylabel)    2 }
  if {$yr<  10} { set Graph(ytick) 0.5 ; set Graph(ylabel)    1 }
}

proc SetLimits { args } {
  global Graph
  set range [split [eval Mag_FindRange $args]]

  set Graph(firstyear) [expr int([lindex $range 0])]
  set Graph(lastyear)  [expr int([lindex $range 1])]
  set Graph(ymin)      [lindex $range 2]
  set Graph(ymax)      [lindex $range 3]

  if {![info exists Graph(xmin)]} { set Graph(xmin) $Graph(firstyear) }
  if {![info exists Graph(xmax)]} { set Graph(xmax) $Graph(lastyear) }

  if {$Graph(ymin)>0.0 && $Graph(ymin)<$Graph(ymax)/2.5} {
    set Graph(ymin) 0
  }

  if {$Graph(initXrange)} {
    set Graph(xmin) 1990
    set Graph(xmax) [expr int($Graph(lastyear))]
    set Graph(initXrange) 0
  }

  ChooseIntervals

  set Graph(ymax) [expr $Graph(ylabel)*int(ceil($Graph(ymax)/$Graph(ylabel)))]
  set Graph(ymin) [expr $Graph(ylabel)*int(floor($Graph(ymin)/$Graph(ylabel)))]

  Mag_GraphLimits $Graph(xmin) $Graph(xmax) $Graph(ymin) $Graph(ymax)
}

proc TimeRangeFrame { buts updproc } {
  global Graph

  if {[winfo exists $buts.timerange]} { destroy $buts.timerange }
  set f [frame $buts.timerange -relief sunken -bd 2]
  pack $f -side top -fill x

#  label $f.title -text "Time range"
  set g [frame $f.grd]
#  label $g.start -text "Start:"
#  label $g.end -text "End:"
  entry $g.estart -width 5 -textvariable Graph(xmin)
  label $g.dash -text "-" -padx 0
  entry $g.eend   -width 5 -textvariable Graph(xmax)
  bind $g.estart <Return> $updproc
  bind $g.eend   <Return> $updproc
#  grid $g.start  -column 1 -row 1 -sticky e
#  grid $g.end    -column 1 -row 2 -sticky e
#  grid $g.estart -column 2 -row 1
#  grid $g.eend   -column 2 -row 2
  pack $g.estart $g.dash $g.eend -side left
  set fy [expr int($Graph(firstyear))]
  set ly [expr int($Graph(lastyear))]
  button $f.b1 -text "1990 - $ly" -pady 0 -command "set Graph(xmin) 1990; set Graph(xmax) $ly;  $updproc"
  button $f.b2 -text "$fy - 1990" -pady 0 -command "set Graph(xmin) $fy;  set Graph(xmax) 1990; $updproc"
  button $f.b3 -text "$fy - $ly"  -pady 0 -command "set Graph(xmin) $fy;  set Graph(xmax) $ly;  $updproc"
  pack $g $f.b1 $f.b2 $f.b3 -side top
}


proc TimeRangeFixed {} {
  global Graph
  set Graph(xmin) [expr int($Graph(firstyear))]
  set Graph(xmax) [expr int($Graph(lastyear))]
  Mag_GraphLimits $Graph(xmin) $Graph(xmax) $Graph(ymin) $Graph(ymax)
}

proc DrawGraticule1 {} {
#
# THIS ONE ISNT USED
#
  global Graph Colour

  set c $Graph(canvas)
  set col $Colour(graticule1)
  set x0 [expr  ceil($Graph(xmin)/$Graph(xtick))*$Graph(xtick)]
  if {$x0==$Graph(xmin)} {incr x0 $Graph(xtick)}
  set x1 [expr floor($Graph(xmax)/$Graph(xtick))*$Graph(xtick)]
  set xscale [expr double($Graph(right)-$Graph(left))/double($Graph(xmax)-$Graph(xmin))]
  set y [expr $Graph(ymin)+$Graph(ytick)]
  set yscale [expr double($Graph(bottom)-$Graph(top))/double($Graph(ymax)-$Graph(ymin))]
  while {$y<$Graph(ymax)} {
    set cy [expr double($Graph(bottom))-double($y-$Graph(ymin))*$yscale]
    set x [expr int($x0)]
    while {$x<$x1} {
      set cx [expr $Graph(left)+($x-$Graph(xmin))*$xscale]
      $c create line [expr $cx-2] $cy [expr $cx] $cy -fill $col
      $c create line $cx [expr $cy-2] $cx [expr $cy] -fill $col
      incr x $Graph(xtick)
    }
    incr y $Graph(ytick)
  }
}

proc DrawGraticule {} {
  global Graph Colour

  set c $Graph(canvas)

  set x0 [expr ceil(double($Graph(xmin))/double($Graph(xtick)))*$Graph(xtick)]
  if {$x0==$Graph(xmin)} {set x0 [expr $x0+$Graph(xtick)]}
  set x1 [expr floor($Graph(xmax)/$Graph(xtick))*$Graph(xtick)]
  set x [expr int($x0)]
  set xscale [expr double($Graph(right)-$Graph(left))/double($Graph(xmax)-$Graph(xmin))]
  while {$x<$x1} {
    set cx [expr $Graph(left)+($x-$Graph(xmin))*$xscale]
    if {($x%$Graph(xlabel))==0} {set col $Colour(graticule1); set tag grat1} else {set col $Colour(graticule2); set tag grat2}
    $c create line $cx $Graph(bottom) $cx $Graph(top) -fill $col -tag $tag
    incr x $Graph(xtick)
  }

  set y [expr $Graph(ymin)+$Graph(ytick)]
  set yscale [expr double($Graph(bottom)-$Graph(top))/double($Graph(ymax)-$Graph(ymin))]
  while {$y<$Graph(ymax)} {
    set cy [expr $Graph(bottom)-($y-$Graph(ymin))*$yscale]
    set wid 0.5
    if {[rmod $y $Graph(ylabel)]==0} {set col $Colour(graticule1); set tag grat1} else {set col $Colour(graticule2); set tag grat2}
    if {($y==0)} {set col black; set wid 1.0}
    $c create line $Graph(left) $cy $Graph(right) $cy -fill $col -tag $tag -width $wid
    set y [expr $y+$Graph(ytick)]
  }

  # make sure light grey lines don't cut through dark grey
  $c lower grat2 grat1
}

proc rmod { a b } {
# like "a%b" but for reals
  set c [expr double($a)/double($b)]
  set c [expr $c-floor($c)]
  set c [expr $c*double($b)]
  return $c
}

proc DrawAxes {} {
  global Graph Font
  set axcol #000000
  set c $Graph(canvas)
  $c create rectangle $Graph(left) $Graph(top) $Graph(right) $Graph(bottom) -outline $axcol

  set x0 [expr  ceil(double($Graph(xmin))/double($Graph(xtick)))*$Graph(xtick)]
  set x1 [expr floor(double($Graph(xmax))/double($Graph(xtick)))*$Graph(xtick)]
  set x [expr int($x0)]
  set xscale [expr double($Graph(right)-$Graph(left))/double($Graph(xmax)-$Graph(xmin))]
  set b1 $Graph(bottom)
  set b2 [expr $b1-3]
  set b3 [expr $b1+2]
  set t1 $Graph(top)
  set t2 [expr $t1+3]
  while {$x<=$x1} {
    set cx [expr $Graph(left)+($x-$Graph(xmin))*$xscale]
#    $c create line $cx $b1 $cx $b2
#    $c create line $cx $t1 $cx $t2
    if {($x%$Graph(xlabel))==0} { $c create text $cx $b3 -anchor n -text $x -font $Font(axes) }
    incr x $Graph(xtick)
  }

  set y $Graph(ymin)
  set yscale [expr double($Graph(bottom)-$Graph(top))/double($Graph(ymax)-$Graph(ymin))]
  set l1 $Graph(left)
  set l2 [expr $l1+3]
  set l3 [expr $l1-3]
  set r1 $Graph(right)
  set r2 [expr $r1-3]
  while {$y<=$Graph(ymax)} {
    set cy [expr $Graph(bottom)-($y-$Graph(ymin))*$yscale]
#    $c create line $l1 $cy $l2 $cy
#    $c create line $r1 $cy $r2 $cy
    if {[rmod $y $Graph(ylabel)]==0} { $c create text $l3 $cy -anchor e -text [expr int($y)] -font $Font(axes) }
    set y [expr $y+$Graph(ytick)]
  }
}

proc MainTitle {txt} {
  global Graph Font
  set x [expr ($Graph(left)+$Graph(right))/2.0]
  $Graph(canvas) create text $x 3 -anchor n -text $txt -font $Font(maintitle)
}

proc ProfileTitles {} {
  global Graph Font
  set x  [expr $Graph(left)+70]
  set y2 [expr $Graph(top)-18]
  set y1 [expr $y2-16]
#  $Graph(canvas) create text $x $y1 -anchor ne -text "Reference: " -font $Font(proftitle)
#  $Graph(canvas) create text $x $y1 -anchor nw -text $Graph(ref)   -font $Font(proftitle)
#  $Graph(canvas) create text $x $y2 -anchor ne -text "Policy: "    -font $Font(proftitle)
#  $Graph(canvas) create text $x $y2 -anchor nw -text $Graph(pol)   -font $Font(proftitle)

  set x [expr ($Graph(left)+$Graph(right))/2.0]
  $Graph(canvas) create text $x $y1 -anchor n -text "Reference: $Graph(ref)" -font $Font(proftitle)
  $Graph(canvas) create text $x $y2 -anchor n -text "Policy: $Graph(pol)" -font $Font(proftitle)
}

# ---------------- Key ---------------------------

proc KeyClear {} {
  global Key
  set Key(dir) {}
}

proc KeyAdd { id txt col sty vis {sym 0}} {
  global Key
  if (![info exists Key(dir)]) {
    set Key(dir) {}
  }
  lappend Key(dir) $id
  set Key($id.txt) $txt
  set Key($id.col) $col
  set Key($id.sty) $sty
  set Key($id.vis) $vis
  set Key($id.sym) $sym
}

proc KeyGap { id } {
  global Key

  lappend Key(dir) $id
  set Key($id.gap) 10
}

proc KeyVis { id vis } {
  global Key
  set Key($id.vis) $vis
}

proc KeyPosition { x1 y1 } {
  global Key
  set Key(posn.x1) $x1
  set Key(posn.y1) $y1
}

proc KeyDisplay {} {
  global Key Graph Font

  $Graph(canvas) delete key

  set x1 $Key(posn.x1)
  set y1 $Key(posn.y1)
  set x2 [expr $x1+20]
  set x3 [expr ($x1+$x2)/2]

  foreach id $Key(dir) {
    if ([info exists Key($id.gap)]) {
      incr y1 $Key($id.gap)
    } else {
      if ($Key($id.vis)) {
        set y2 [expr $y1+10]
        set y3 [expr $y1+5]
        switch $Key($id.sty) {
          range1 { Mag_HatchRect key $x1 $y1 $x2 $y2 $Key($id.col) 2 4 }
          range2 { Mag_HatchRect key $x1 $y1 $x2 $y2 $Key($id.col) 1 4 }
#          range1 { $Graph(canvas) create rectangle $x1 $y1 $x2 $y2 -fill $Key($id.col) -outline "" -stipple @poldot.xbm -tag key }
#          range2 { $Graph(canvas) create rectangle $x1 $y1 $x2 $y2 -fill $Key($id.col) -outline "" -stipple @refdot.xbm -tag key }
          solid  { $Graph(canvas) create line      $x1 $y3 $x2 $y3 -fill $Key($id.col) -width 2 -tag key }
        }
        if {$Key($id.sym)!=0} {
          Mag_DrawSymbol $x3 $y3 $Key($id.sym) $Key($id.col) key
        }
        $Graph(canvas) create text [expr $x1+25] $y3 -anchor w -text $Key($id.txt) -font $Font(key) -tag key
        incr y1 15
      }
    }
  }
}

# -------

proc SwitchVisInit { frm } {
  global Switch
  set Switch(frame) $frm.switch
  if {[winfo exists $Switch(frame)]} {
    destroy $Switch(frame)
  }
  frame $Switch(frame) -relief sunken -bd 2
  pack $Switch(frame) -side top -anchor w -fill x
}

proc SwitchVisSet { curve keyid txt } {
  global Switch Key
  checkbutton $Switch(frame).$keyid -text $txt -pady 0 -variable Key($keyid.vis) -command "SwitchPressed $curve $keyid"
  pack $Switch(frame).$keyid -side top -anchor w
}

proc SwitchPressed { curve keyid } {
  global Graph Switch Key

  # move off & on visible region. I can't find a "-visible" command.
  if {$Key($keyid.vis)} {
    $Graph(canvas) move $curve 0 -1000
  } else {
    $Graph(canvas) move $curve 0 1000
  }
  KeyDisplay
}

# --------------------------------------------

proc Trc { name index op } {
  upvar $name var
  if {$index==""} {
    puts "$op $name = $var"
  } else {
    puts "$op $name ($index) = $var($index)"
  }
}

