# print.tcl - handle printing


proc oldPrintGraph {can} {
  global Font Colour
  set colmap(grey) { 1.0 1.0 1.0 setrgbcolor }
  set colmap($Colour(graticule1)) { 0.8  0.8  0.8  setrgbcolor }
  set colmap($Colour(graticule2)) { 0.9  0.9  0.9  setrgbcolor }
  set fontmap($Font(axes))      { Helvetica   10 }
  set fontmap($Font(maintitle)) { Times-Roman 20 }
  set fontmap($Font(proftitle)) { Times-Roman 10 }
  set fontmap($Font(key))       { Times-Roman 10 }
  $can postscript -colormap colmap -file magicc.ps -fontmap fontmap -pagewidth 7.5i
}

proc UnixPrintGraph {can} {
  global PrintCommand env
  global TitleBanner
  toplevel .print
  wm title .print "$TitleBanner - Print Graph"
  append cmd "lpr -P" $env(PRINTER)
  set PrintCommand $cmd
  label .print.txt -text "Enter a command that will read\na PostScript file from stdin and\nprint it to the desired printer." -padx 10 -pady 10
  entry .print.cmd -textvariable PrintCommand
  set f [frame .print.buts]
  pack .print.txt .print.cmd -side top
  pack $f -side top -pady 10
  button $f.okay -text "OK" -command "DoPrintGraph $can; destroy .print"
  button $f.cancel -text "Cancel" -command {destroy .print}
  pack $f.okay $f.cancel -side left
}

proc PrintMappings {} {
  global Font Colour Fontmap Colmap
  set Colmap(grey) { 1.0 1.0 1.0 setrgbcolor }
  set Colmap($Colour(graticule1)) { 0.8  0.8  0.8  setrgbcolor }
  set Colmap($Colour(graticule2)) { 0.9  0.9  0.9  setrgbcolor }
  set Fontmap($Font(axes))      { Helvetica   10 }
  set Fontmap($Font(maintitle)) { Times-Roman 18 }
  set Fontmap($Font(proftitle)) { Times-Roman 12 }
  set Fontmap($Font(key))       { Times-Roman 12 }
}

proc DoPrintGraph {can} {
  global Fontmap Colmap PrintCommand
  PrintMappings
  append file [pid] ".ps"
  $can postscript -colormap Colmap -file $file -fontmap Fontmap -pagewidth 7.5i
  update
  eval "exec $PrintCommand < $file"
  exec rm $file
}

proc EPSfile {can} {
  global Print
  global TitleBanner
  PrintMappings
  set Print(can) $can
  set tl [toplevel .epsfile]
  wm title .epsfile "$TitleBanner - Print to EPS file"
  label $tl.l1 -text "Enter filename to contain Encapsulated PostScript" -padx 10 -pady 10
  entry $tl.fname -textvariable Print(fname) -width 30
  set f [frame $tl.buts]
  pack $tl.l1 $tl.fname -side top
  pack $f -side top -pady 10
  button $f.okay -text "OK" -command {
    if {$Print(fname)!=""} {
      $Print(can) postscript -file $Print(fname) -colormap Colmap -fontmap Fontmap -pagewidth 7.5i
      destroy .epsfile
    }
  }
  button $f.cancel -text "Cancel" -command "destroy $tl"
  pack $f.okay $f.cancel -side left
  focus $tl.fname
}

proc PrintGraph { can } {
  global tcl_platform
  switch $tcl_platform(platform) {
    unix    { UnixPrintGraph $can }
    windows { print_canvas $can }
  }
}

