# run.tcl - runs model

proc RunModel {} {
  global File User Default RunCancel
  global TitleBanner
  if {[winfo exists .run]} {
    focus .run
    return
  }
  toplevel .run
  wm title .run "$TitleBanner - running model"
  set f [frame .run.a]
  pack $f -side top -fill x
  label $f.a -text "Policy/User:"
  set f1 [label $f.b -width 15]
  pack $f.b $f.a -side right
  set f [frame .run.b]
  pack $f -side top -fill x
  label $f.a -text "Policy/Default:"
  set f2 [label $f.b -width 15]
  pack $f.b $f.a -side right
  set f [frame .run.c]
  pack $f -side top -fill x
  label $f.a -text "Reference/User:"
  set f3 [label $f.b -width 15]
  pack $f.b $f.a -side right
  set f [frame .run.d]
  pack $f -side top -fill x
  label $f.a -text "Reference/Default:"
  set f4 [label $f.b -width 15]
  pack $f.b $f.a -side right
  button .run.cancel -text "Cancel\n(after this segment)" -command {set RunCancel 1}
  pack .run.cancel -side bottom -fill x
  set RunCancel 0
  bind .run.cancel <Button-1> {set RunCancel 1}
  # bind as well, because doesn't seem too keen to see first click

  grab .run

  # make a local copy of Default and overwrite output parameters

  array set def [array get Default]
  set def(kyrref)   $User(kyrref)
  set def(iy0)      $User(iy0)
  set def(lastyear) $User(lastyear)
  set def(itemprt)  $User(itemprt)
  set def(iusergas) 0

  CopyFile $File(policy).gas gas.emk

  update
  if {!$RunCancel} {
    WriteControlFile User
CopyFile maguser.cfg maguser.up
    $f1 configure -text "running..."
    update
    RunFortran
  }
  update
  if {!$RunCancel} {
    $f1 configure -text "collating..."
    update
    CollectOutput up
    $f1 configure -text "done."
    update
  }

  update
  if {!$RunCancel} {
    WriteControlFile def
CopyFile maguser.cfg maguser.dp
    $f2 configure -text "running..."
    update
    RunFortran
  }
  update
  if {!$RunCancel} {
    $f2 configure -text "collating..."
    update
    CollectOutput dp
    $f2 configure -text "done."
    update
  }

  CopyFile $File(reference).gas gas.emk

  update
  if {!$RunCancel} {
    WriteControlFile User
CopyFile maguser.cfg maguser.ur
    $f3 configure -text "running..."
    update
    RunFortran
   }
  update
  if {!$RunCancel} {
   $f3 configure -text "collating..."
    update
    CollectOutput ur
    $f3 configure -text "done."
    update
  }

  update
  if {!$RunCancel} {
    WriteControlFile def
CopyFile maguser.cfg maguser.dr
    $f4 configure -text "running..."
    update
    RunFortran
  }
  update
  if {!$RunCancel} {
    $f4 configure -text "collating..."
    update
    CollectOutput dr
    $f4 configure -text "done."
    update
  }

  destroy .run
}

proc CollectOutput {run} {
  CopyFile mag.out      report.$run
  CopyFile concs.dis    concs.$run
  CopyFile emiss.dis    emiss.$run
  CopyFile forcings.dis forcings.$run
  CopyFile sealev.dis   sealev.$run
  CopyFile temps.dis    temps.$run
  CopyFile hidrive.out  sgh.$run
  CopyFile middrive.out sgm.$run
  CopyFile lodrive.out  sgl.$run
  CopyFile usrdrive.out sgu.$run
}

# New control file for magtar.exe; other parameters are defined in
# various magfoo.cfg files, not editable through GUI.

proc WriteControlFile {arrname} {

  upvar $arrname arr
  set f [open maguser.cfg w]

  puts $f [format "%7d"    [expr $arr(ccm)      ]] nonewline
  puts $f "    *     LEVCO2   : 1,2,3,4 = LO,MID,HIGH,USER CO2"
  puts $f [format "%7d"    [expr $arr(cccf)     ]] nonewline
  puts $f "    *     IFEED    : IF=0, TURNS OFF CLIMATE FEEDBACK ON CARBON CYCLE"
  puts $f [format "%7d"    [expr $arr(aerosol)  ]] nonewline
  puts $f "    *     LEVSO4   : 1,2,3,4 = LO,MID,HIGH,USER 1990 SO4 FORCING"
  puts $f [format "%8.1f"  [expr $arr(dt2xuser) ]] nonewline
  puts $f "   *     DT2XUSER : USER VALUE OF DT2X IN CLIMATE MODEL LOOP"
  puts $f [format "%8.1f"    [expr $arr(kzuser)      ]] nonewline
  puts $f "   *     YK       : USER VALUE OF VERT. DIFF. (cm**2/sec) (TAR=2.3)"
  puts $f [format "%7d"    [expr $arr(thc)      ]] nonewline
  puts $f "    *     IVARW    : 0 = CONST, 2 = VBLE(RIGHT), 1 = VBLE(WRONG)"
  puts $f [format "%7d"    [expr $arr(model)    ]] nonewline
  puts $f "    *     MODEL    : =0, USES USER PARAMS; ELSE= HARDWIRED OAGCM"
  puts $f [format "%7d"    [expr $arr(kyrref)   ]] nonewline
  puts $f "    *     KYRREF   : REFERENCE YEAR FOR CLIMATE MODEL OUTPUT"
  puts $f [format "%7d"    [expr $arr(iy0)      ]] nonewline
  puts $f "    *     IY0      : FIRST YEAR FOR CLIMATE MODEL OUTPUT"
  puts $f [format "%7d"    [expr $arr(lastyear) ]] nonewline
  puts $f "    *     LASTYEAR : LAST YEAR FOR CLIMATE MODEL RUN"
  puts $f [format "%7d"    [expr $arr(itemprt)  ]] nonewline
  puts $f "    *     ITEMPRT  : PRINTOUT INTERVAL FOR CLIMATE MODEL"
  puts $f [format "%7d"    [expr $arr(ice) ]] nonewline
  puts $f "    *     ICE      : =1,2,3 USES LOW,MID,HIGH ICE MELT IN USER CASE"


  puts $f "1234567890 *"
  puts $f "      : LEVCO2  : 1,2,3,4 USES DUSER=0.4,1.1,1.8,DUSER"
  puts $f "      : LEVSO4  : =1 USES S90DIR,S90IND,S90BIO,FOC90 = -0.3,-0.4,-0.2,0.0"
  puts $f "                : =2 USES S90DIR,S90IND,S90BIO,FOC90 = -0.4,-0.8,-0.2,0.1"
  puts $f "                : =3 USES S90DIR,S90IND,S90BIO,FOC90 = -0.5,-1.2,-0.2,0.2"
  puts $f "                : =4 USES USER VALUES OF S90DIR,S90IND,S90BIO,FOC90"
  puts $f "      : KEYDW SCALING"
  puts $f "          METHOD 1 USES DW = W0*(T/TW0), 2 USES DW = AW*W0*(T/TW0)"
  puts $f "           WHERE AW IS THE 'ACTIVE' PART OF W (0.3 IN TAR):"
  puts $f "           KEYDW=1; METHOD 1, T= TGLOBE"
  puts $f "           KEYDW=2; METHOD 1, T= TOCEAN"
  puts $f "           KEYDW=3; METHOD 1, T= HEMISPHERIC TOCEAN"
  puts $f "           KEYDW=4; METHOD 2, T= TGLOBE  (THIS IS WHAT TAR USES)"
  puts $f "           KEYDW=5; METHOD 2, T= TOCEAN"
  puts $f "          ACTIVE W OPTION ONLY APPLIED IF IVARW=2."
  puts $f "C"
  puts $f "	 : MODELS ACCESSIBLE"
  puts $f "	     0 = MAGICC USER MODEL"
  puts $f "	     1 = GFDL"
  puts $f "	     2 = CSIRO"
  puts $f "	     3 = HadCM3"
  puts $f "	     4 = HadCM2"
  puts $f "	     5 = ECH4/OPYC"
  puts $f "	     6 = CSM"
  puts $f "	     7 = PCM"
  puts $f "C"

  close $f

CopyFile maguser.cfg [string tolower $arrname].mu
}


proc CopyFile {source dest} {
  global tcl_platform
  switch $tcl_platform(platform) {
    unix { exec cp $source $dest }
    windows { SlowCopyFile $source $dest }
  }
}

proc SlowCopyFile {source dest} {
  set in  [open $source r]
  set out [open $dest   w]
  while { [gets $in line] >= 0 } {
    puts $out $line
  }
  close $in
  close $out
}

proc RunFortran {} {
  global tcl_platform
  switch $tcl_platform(platform) {
    unix { 
	exec ./magtar 
    }
    windows { 
	exec ./magtar.exe
    }
  }
}

