# wprint.tcl - platform-specific printing for MS-Windows

load wprint

proc prcoord {coords n} {
  # returns the nth item of coords (starting at 1) scaled for GDI
  return [expr int(10.0*[lindex $coords [expr $n-1]])]
}

proc print_canvas_elements {h c} {
# uses GDI calls to copy all elements of a canvas to an already opened
# Windows graphical device (eg. printer, metafile)

  # calculate point -> world coord factor
  set scrmmhgt [winfo screenmmheight $c]
  set scrhgt [winfo screenheight $c]
  # 1.5 is fudge factor!
  set unitsperpoint [expr ($scrmmhgt*28.35)/$scrhgt*1.5]

  # calculate unit pen width
  set xmax [expr 10*[winfo width  $c]]
  set x [Wprint_GetDeviceCaps $h HORZRES]
  set unitpenwidth [expr $x/($xmax/10.0)]

  set hpen [Wprint_CreatePen SOLID [expr round($unitpenwidth)] 0]
  Wprint_SelectObject $h $hpen
  set hbrush [Wprint_CreateBrush HOLLOW 0]
  Wprint_SelectObject $h $hbrush
  set hfont [Wprint_CreateFont "Times New Roman" [expr $unitsperpoint*10]]
  Wprint_SelectObject $h $hfont

  Wprint_SetBkMode $h TRANSPARENT

  # anchor lookup table
  array set anchorlut "nw 9 n 8 ne 7 w 6 center 5 e 4 sw 3 s 2 se 1"

  # iterate across canvas items
  foreach n [$c find all] {
    set type [$c type $n]
    set coords [$c coords $n]
    set cfg [$c itemconfigure $n]

    set width   1
    set fill    0
    set outline 0
    set font    "Times New Roman"
    set point  10
    set text    ""
    set anchor  5

    foreach cf $cfg {
      set opt [lindex $cf 0]
      set val [lindex $cf 4]
      switch [string trim $opt -] {

        "width" {
          set width $val
        }

        "fill" {
          if {$val=={}} {
            set fill $val
          } else {
            set fill [Wprint_TkGetColor $val]
          }
        }

        "outline" {
          if {$val=={}} {
            set outline $val
          } else {
            set outline [Wprint_TkGetColor $val]
          }
        }

        "font" {
          switch [lindex $val 0] {
            "Times" { set font "Times New Roman" }
            "Arial" { set font "Arial" }
          }
          set point [lindex $val 1]
        }

        "anchor" {
          set anchor $anchorlut($val)
        }

        "text" {
          set text $val
        }

      }
    }

    switch $type {
      line {
        if {$fill!={}} {
          set newpen [Wprint_CreatePen SOLID [expr round($width*$unitpenwidth)] $fill]
          Wprint_DeleteObject $hpen
          set hpen $newpen
          Wprint_SelectObject $h $hpen
          Wprint_MoveTo $h [prcoord $coords 1] [prcoord $coords 2]
          set n [llength $coords]
          for {set i 3} {$i<$n} {incr i 2} {
            Wprint_LineTo $h [prcoord $coords $i] [prcoord $coords [expr $i+1]]
          }
        }
      }
      rectangle {
        if {$fill=={}} {
          set newbrush [Wprint_CreateBrush HOLLOW 0]
        } else {
          set newbrush [Wprint_CreateBrush SOLID $fill]
        }
        Wprint_SelectObject $h $newbrush
        Wprint_DeleteObject $hbrush
        set hbrush $newbrush
        if {$outline=={}} {
          set newpen [Wprint_CreatePen NULL 0 0]
        } else {
          set newpen [Wprint_CreatePen SOLID [expr round($width*$unitpenwidth)] $outline]
        }
        Wprint_SelectObject $h $newpen
        Wprint_DeleteObject $hpen
        set hpen $newpen
        Wprint_Rectangle $h \
          [prcoord $coords 1] [prcoord $coords 2] \
          [prcoord $coords 3] [prcoord $coords 4]
      }
      text {
#        puts "text \"$text\" ( $font $point $anchor $fill )"
        if {$fill!={}} {
          set newpen [Wprint_CreatePen SOLID [expr round($width*$unitpenwidth)] $fill]
          Wprint_DeleteObject $hpen
          set hpen $newpen
          Wprint_SelectObject $h $hpen
          Wprint_DeleteObject $hfont
          set hfont [Wprint_CreateFont $font [expr $point*$unitsperpoint]]
          Wprint_SelectObject $h $hfont
          Wprint_TextOut $h [prcoord $coords 1] [prcoord $coords 2] $text $anchor
        }
      }
      oval {
#puts "oval: h=$h  fill=$fill  outline=$outline"
        if {$fill=={}} {
#puts "  hollow"
          set newbrush [Wprint_CreateBrush HOLLOW 0]
        } else {
#puts "  solid"
          set newbrush [Wprint_CreateBrush SOLID $fill]
        }
        Wprint_SelectObject $h $newbrush
        Wprint_DeleteObject $hbrush
        set hbrush $newbrush
        if {$outline=={}} {
          set newpen [Wprint_CreatePen NULL 0 0]
        } else {
          set newpen [Wprint_CreatePen SOLID [expr round($width*$unitpenwidth)] $outline]
        }
        Wprint_SelectObject $h $newpen
        Wprint_DeleteObject $hpen
        set hpen $newpen

#set x [Wprint_GetCurrentObject $h PEN]
#puts "pen $x:"
#puts "  [Wprint_GetObject $x]"
#set x [Wprint_GetCurrentObject $h BRUSH]
#puts "brush $x:"
#puts "  [Wprint_GetObject $x]"

        Wprint_Ellipse $h \
            [prcoord $coords 1] [prcoord $coords 2] \
            [prcoord $coords 3] [prcoord $coords 4]
      }
      polygon {
        if {$fill=={}} {
          set newbrush [Wprint_CreateBrush HOLLOW 0]
        } else {
          set newbrush [Wprint_CreateBrush SOLID $fill]
        }
        Wprint_SelectObject $h $newbrush
        Wprint_DeleteObject $hbrush
        set hbrush $newbrush
        if {$outline=={}} {
          set newpen [Wprint_CreatePen NULL 0 0]
        } else {
          set newpen [Wprint_CreatePen SOLID [expr round($width*$unitpenwidth)] $outline]
        }
        Wprint_SelectObject $h $newpen
        Wprint_DeleteObject $hpen
        set hpen $newpen
        Wprint_PolyInit
        set n [llength $coords]
        for {set i 3} {$i<$n} {incr i 2} {
          Wprint_PolyAdd [prcoord $coords $i] [prcoord $coords [expr $i+1]]
        }
        Wprint_PolyDraw $h
      }

    }

  }
}


proc print_canvas {c} {
  # init printer
  set h [Wprint_PrintDlg]
  if {$h==0} {return}
  Wprint_StartDoc $h Test
  Wprint_StartPage $h

  # set coordinate system to 0.1 pixel steps
  set xmax [expr 10*[winfo width  $c]]
  set ymax [expr 10*[winfo height $c]]
  Wprint_SetMapMode $h ISOTROPIC
  Wprint_SetWindowExt $h $xmax $ymax

  # draw across full page
  set x [Wprint_GetDeviceCaps $h HORZRES]
  set y [Wprint_GetDeviceCaps $h VERTRES]
  Wprint_SetViewportExt $h $x $y

  print_canvas_elements $h $c

  # close down printer
  Wprint_EndPage $h
  Wprint_EndDoc $h
}

proc create_wmf {c fname} {
# creates a Windows Metafile

  set h [Wprint_CreateMetaFile $fname]
  if {$h==0} {return}

  # set coordinate system to 0.1 pixel steps
  set xmax [expr 10*[winfo width  $c]]
  set ymax [expr 10*[winfo height $c]]
  Wprint_SetMapMode $h ISOTROPIC
  Wprint_SetWindowExt $h $xmax $ymax

  print_canvas_elements $h $c

  Wprint_CloseMetaFile $h
}

proc wprint_text { txt nchars lmarg rmarg tmarg bmarg } {
# prints text in "txt" using a font suitable to allow "nchars" columns
# across the page, allowing for "lmarg" millimetres left margin, "rmarg"
# right margin, "tmarg" top margin and "bmarg" bottom margin.

  # init printer
  set h [Wprint_PrintDlg]
  if {$h==0} {return}
  Wprint_StartDoc $h Test
  Wprint_StartPage $h
  Wprint_SetMapMode $h LOMETRIC

  # get device size & calc box size
  set dsx [Wprint_GetDeviceCaps $h HORZSIZE]
  set dsy [Wprint_GetDeviceCaps $h VERTSIZE]
  set boxx [expr $dsx-($lmarg+$rmarg)]
  set boxy [expr $dsy-($tmarg+$bmarg)]

  # calc space for one character, then point size, then create font
  set charx [expr double($boxx)/double($nchars)]
  set hfont [Wprint_CreateFont "Courier New" [expr int(floor($charx*17.7))]]
  Wprint_SelectObject $h $hfont

  set ytop [expr -$tmarg*10]
  set ybot [expr ($bmarg-$dsy)*10]

  set x [expr $lmarg*10]
  set y $ytop
  set ysize -[lindex [Wprint_GetTextExtent $h "M"] 1]

  foreach line [split $txt \n] {
    if {($y+$ysize)<$ybot} {
      Wprint_EndPage $h
      Wprint_StartPage $h
      Wprint_SetMapMode $h LOMETRIC
      Wprint_SelectObject $h $hfont
      set y $ytop
    }
    Wprint_TextOut $h $x $y $line 9
    incr y $ysize
  }

  Wprint_EndPage $h
  Wprint_EndDoc $h
}
