#!/tcl/bin/wish41
#!/usr/bin/wish -f
# 19/03/97 cr_map and save21.exe changed for cloud cover
# Version 2.1 with new save21.exe file (cr_map.exe the same)
# 2.1a all Mike's comments from 05/03/97
# Combine UNEP and general version 24/02/97
# Ver 2.3 added UIUC model (GHG+SO4)  18/08/97
#
# 01/01/2000
# Ver 2.4 added 6 models + HadCM2 changed for HGGX
# reference year 2000 + some minor corrections

# 08/01/02002
# New development by Seth McGinnis begins with Ver 3.0

# 03/21/02003
# Jump to version 4.0 for beta-testing release.

# The Param array keeps lots of state

set Param(Version) "4.1"
set Param(Verdate) "September 2003"
set Param(aerosol) 0
set Param(contour) default
set Param(definition) 1
set Param(deltaT) 0
set Param(engineflag) true
set Param(gas) d
set Param(interval) 2050
set Param(mask) A
set Param(magicc) u
set Param(mapnumber) -1
set Param(outname) ""
# set Param(region) world
set Param(reversecolor) no
set Param(scaling) linear
set Param(scenario) ""

set Param(season) Annual
set Param(seas) Ann
set Param(observable) "Mean Temp"
set Param(obs) temp
set Param(analysis) change
set Param(units) "deg C"


# Variable synchronization section: the "trace variable" command and
# SynchFoo functions are used to automagically update related Param
# variables whenever the primary form changes.


# Seasons: long form for annotations and short (three-letter)
# abbreviation used on buttons.  Season is primary, seas is secondary.

set Seasons {January February March April May June July \
        August September October November December \
        Dec/Jan/Feb Mar/Apr/May Jun/Jul/Aug Sep/Oct/Nov Annual}

set Seas {Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec \
        DJF MAM JJA SON Ann}

proc SynchSeas {vname dummy op} {
    global Param Seas Seasons
    set Param(seas) [lindex $Seas [lsearch -exact $Seasons $Param(season)]]
}

trace variable Param(season) w SynchSeas


# "Param(observable)" is generally used for display purposes, while
# "Param(obs)" is used mostly for array indexing and filename
# suffixes.  (Array names are a big pain if the name has a space in
# it...)

set Observables {{Mean Temp} {Precipitation}}
set Obs {temp prec}

proc SynchObs {vname dummy op} {
    global Param Obs Observables
    set i [lsearch -exact $Observables $Param(observable)]
    set Param(obs) [lindex $Obs $i]
}

trace variable Param(observable) w SynchObs


# The Analyses array defines the different kinds of data analysis that
# SCENGEN can display.  These are also the legal values that
# Param(analysis) can take on, and are suitable for indexing, button
# names, etc.  The associated arrays are Annames, a short name
# suitable for button titles; Anphrases, a long spelled-out version;
# and Anpreps, a preposition you can use to append the observable to
# the phrase and get something suitable for annotations. (Note: one
# might also use Anphrases for hover popups on the analysis buttons,
# if one were motivated, which is why we do the prepositions
# separately.)  Finally, the Datafile array names the affiliated data
# file in ../engine/scengen.  These arrays are all used statically
# (i.e, they aren't watched), so we don't have to do any Param
# synchronization.

set Analyses {change error imsnr modbase modchange obsbase obschange \
        probincr sdbase sdchange tempsnr}

set Annames {{Change} {Error} {Inter-SNR} {Mod. Base.} {Mod.+Change} \
        {Obs. Base.} {Obs.+Change} {P(Increase)} {S.D. Base.} \
        {S.D. Change} {Tempor. SNR}}

set Anphrases {{Change} {Model Error (Model - Observed)} \
        {Inter-model Signal-to-Noise Ratio} {Model-Average Baseline} \
        {Actual (Change + Modeled)} {Observed Baseline} \
        {Actual (Change + Observed)} {Probability of Increase} \
        {Variability (Std. Dev.)}  {Change in Variability} \
        {Temporal Signal-to-Noise Ratio}}

set Anpreps { in in for {} {} {} {} in of of for }

set Datafiles {DEL2USE.OUT ERROR.OUT IM-SNR.OUT MODBASE.OUT \
        ABS-MOD.OUT OBSBASE.OUT ABS-OBS.OUT PROBINCR.OUT \
        BASE-SD.OUT DELTA-SD.OUT BAROFSNR.OUT}


# Units are complicated, because they depend both on what is observed
# and on how it is analyzed.  SynchUnits watches both Param(analysis)
# and Param(obs).  (Note: P(obs) is downstream of P(observable); make
# sure to watch the variables you're using in the function, not their
# primaries.)

proc SynchUnits {vname dummy op} {
    global Param

    switch $Param(obs) {
        temp {set Param(units) {deg C}}
        prec {
            set Param(units) {mm/day}
            if {$Param(analysis) == "change" || $Param(analysis) == "error" } {
                set Param(units) {%}
            }
        }
        default {
            Error "Observable with unknown units!"
            set Param(units) {}
        }
    }
    
    switch $Param(analysis) {
        sdchange {set Param(units) {%} }
        imsnr -
        tempsnr -
        probincr {set Param(units) {} }
        default {}
    }
}    

trace variable Param(obs) w SynchUnits
trace variable Param(analysis) w SynchUnits


# Certain of the precipitation cases should have a reversed colormap,
# with blue high and red low.  The cases are defined in contours.tcl
# and set using a trace function on Param(analysis) in a general way
# here.  Note that this only sets the default when the analysis
# changes; it is still possible to invert the contour order manually.

proc SynchReverse {vname dummy op} {
    global Param contourReverseColor

    set Param(reversecolor) $contourReverseColor($Param(obs),$Param(analysis))
}

trace variable Param(obs) w SynchReverse
trace variable Param(analysis) w SynchReverse


# Mapping parameters.  mapBdy determines the region displayed,
# gridsize(can) is the number of pixels per map tile, and
# gridsize(map) is the number of degrees per map tile.  Naturally, the
# map will be (mapBdy(e)-mapBdy(w))*gridsize(can)/gridsize)map) pixels
# in width.

set mapBdy(n) 90.0
set mapBdy(e) 180.0
set mapBdy(s) -90.0
set mapBdy(w) -180.0

set gridsize(can) 8
set gridsize(map) 5

set boxAnchor(lat) 90
set boxAnchor(lon) -180
set boxLast(lat) -90
set boxLast(lon) 180


# gcms.tcl contains information about the models used.  Defines
# gcmPath, gcmList, and the gcmPreset() array.

source gcms.tcl


# load system-dependent library resources

switch $tcl_platform(platform) {
    unix { load wldmap.so wldmap }
    windows { load wldmap.dll wldmap }
    default { puts "Unknown platform: $tcl_platform(platform)" ; exit }
}
switch $tcl_platform(platform) {
    unix    { load tile.so tile }
    windows { load tile.dll tile }
    default { puts "Unknown platform: $tcl_platform(platform)" ; exit }
}


# load user-settable look-and-feel parameters

source font.tcl
source contours.tcl

source html_lib.tcl
source help.tcl

wm withdraw .
after 0 Splash

### That's the end of (main).  Everything after this point is a
### function.  


# Create splash screen

proc Splash {} {
    global Font Param

    set s [toplevel .splash]
    wm title $s "SCENGEN $Param(Version)"

    # title stuff

    set f [frame $s.title -height 100]
    pack $f -side top -fill x

    label $f.uea -bitmap @uea.xbm -fg blue
    label $f.cru -bitmap @cru.xbm -fg blue
    label $f.title -text "SCENGEN" -font $Font(sgtitle) -fg blue

    pack $f.uea -side left  -anchor n -padx 10 -pady 32
    pack $f.cru -side right -anchor n -padx 10 -pady 32
    pack $f.title -side top -pady 22

    label $s.subtitle -font $Font(sgname) -fg blue \
            -text "A Global and Regional Climate Change Scenario Generator"
    pack $s.subtitle -side top -pady 2m -padx 10m
    
    # ncar and stratus logos to either side of the info block

    set f [frame $s.info]
    pack $f -side top -fill x
    
    label $f.ncar -bitmap @ncar.xbm -fg blue
    pack $f.ncar -side left -padx 10
    label $f.stratus -bitmap @stratus.xbm -fg blue
    pack $f.stratus -side right -padx 10

    # information block (authors)    

    set f [frame $s.info.authors]
    pack $f -side top -pady 3m

    label $f.author1 -text "Concept and Scientific Programming: T.M.L. Wigley"
    label $f.author2 -text \
            "Design: T.M.L. Wigley, M. Hulme, M. Salmon, S. McGinnis"
    label $f.author3 -text "User Interface: S. McGinnis, M. Salmon"
    label $f.author4 -text "Data Set Development: C. Doutriaux"
    label $f.author5 -text "Other Contributers: O. Brown, T. Jiang, \
            \nP.D. Jones, M. New, B.D. Santer"
    label $f.version -text "Version $Param(Version), $Param(Verdate)" \
            -bg white -fg blue

    pack $f.author1 $f.author2 $f.author3 $f.author4 $f.author5 -side top
    pack $f.version -side top -pady 8m

    # OK button kills the splash screen, runs a bunch of initializer
    # functions, and creates the initial control windows.
    
    frame $s.but 
    pack $s.but -side bottom -fill x
    button $s.but.ok -text "OK" -width 7 -command {
        destroy .splash
        PickScenario
        ControlWindow
        MapWindow
        UseGCMPreset Default
        UpdateDeltaT
    }
    pack $s.but.ok -side bottom -expand on -pady 3m
}


# reads in from file the names of the policy and reference scenarios.

proc PickScenario {} {
    global Param
    
    set magicc_file "../engine/sgu.dp" 
    set f [open $magicc_file]
    gets $f sc_name
    scan $sc_name "%8s %8s" hdr Param(policy)
    close $f
    set magicc_file "../engine/sgu.dr"
    set f [open $magicc_file]
    gets $f sc_name
    scan $sc_name "%8s %8s" hdr Param(reference)
    close $f
    set Param(scenario) $Param(reference)
}


# The toplevel . window becomes ControlWindow; it's the absolutely
# essential one, and that way if you close it, the entire program
# dies.  The deiconify command brings it back from the "withdraw"
# command up in main.

proc ControlWindow {} {
    global Font Param
    
    set control .
    wm title $control "SCENGEN"
    wm geometry $control +10+10
    wm deiconify .

    set f [frame $control.windows -bd 1 -relief groove]
    pack $f -side left -anchor n
    WindowCtl $f

    set f [frame $control.actions -bd 1 -relief groove]
    pack $f -side right -anchor n
    ActionCtl $f
}


proc ActionCtl {f} {
    global Param

    label $f.title -text "Actions"
    button $f.quit -text "QUIT" -command {SGExit}
#    button $f.print -text "Help" -command {SGHelp}
#    button $f.help -text "Help" -command {
#       Error "No help is currently available for version $Param(Version)."
#    }
button $f.help -text "Help" -command {Help index.htm}
#    button $f.print -text "Print" -command {PrintMap}
    button $f.print -text "Print" -command {
        Error "Printing is currently disabled in version $Param(Version)."
    }
    button $f.view -text "RUN" -command {MapWindow; ViewMap}

    frame $f.fund
    if {[file exists "../fund"]} {
        source ../fund/fund.tcl
        button $f.fund.button -text "FUND data" -command {
            FundDialog
        }
        pack $f.fund.button -side top
    }

    pack $f.title $f.quit $f.help $f.print $f.fund $f.view \
            -side top -fill x
}


proc WindowCtl {f} {

    label $f.title -text "Control Windows"

    button $f.analysis -text "Analysis" -command {AnalysisWindow}
    button $f.gcms -text "Models" -command {GCMWindow}
    button $f.region -text "Region" -command {RegionWindow}
    button $f.variable -text "Variable" -command {VariableWindow}
    button $f.warming -text "Warming" -command {WarmingWindow}

    pack $f.title $f.analysis $f.gcms $f.region $f.variable $f.warming \
            -side top -fill x

}


# Creates a new map window; any old map windows remain but become
# static.  (Not printable, no dynamic annotation.)

proc MapWindow {} {
    global Font Param can map mapBdy

    incr Param(mapnumber)

    if {$Param(mapnumber) == 1} {
        wm title $map "Map $Param(mapnumber)"
    } else {

        # disable old dynamic annotations; they depend on data which gets
        # overwritten when the new map is drawn.
        if {$Param(mapnumber) > 1} {pack forget $map.dynannot}
        
        set map [toplevel .map$Param(mapnumber)]
        wm title $map "Map $Param(mapnumber)"
        wm geometry $map +10+380
        
        set f [frame $map.dynannot]
        pack $f -side bottom -anchor w -fill x
        DynamicAnnotations $f
        
        set can [canvas $map.can -borderwidth 0 -highlightthickness 0 \
                -background gray \
                -width [expr {200 + [MapWidth]+1}] \
                -height [expr {[MapHeight] +1}]]
        pack $can -side top -anchor w
        PutMap
    }
    if {$Param(mapnumber) == 0} {
        wm title $map "Map"
    }
}


# The textvariables get bound to mouse events in callbacks that update
# them as the pointer moves across the map.

proc DynamicAnnotations {f} {
    global Param Font Lat Lon Val
    
    set lat [frame $f.lat]
    set lon [frame $f.lon]
    set val [frame $f.val]
    
    label $lat.name -text "Latitude:"
    label $lat.val -textvariable Lat -fg blue -width 15 -anchor w
    label $lon.name -text "Longitude:"
    label $lon.val -textvariable Lon -fg blue -width 15 -anchor w
    label $val.name -text "Value:"
    label $val.val -textvariable Val -fg blue -width 15 -anchor w
    
    pack $lat.name $lat.val -side left
    pack $lon.name $lon.val -side left
    pack $val.name $val.val -side left
    
    pack $lat $lon $val -side left
}



# Controls for the model parameters that affect the global
# temperature.

proc WarmingWindow {} {
    if {[winfo exists .warm]} {
        focus .warm
        raise .warm
    } else {
        set w [toplevel .warm]
        wm title .warm "Warming"
        
        set f [frame .warm.main]
        pack $f -side top -fill both -expand on
        WarmCtl $f
    }
}


proc WarmCtl {t} {

    frame $t.indicator -bd 1 -relief groove
    DeltaTIndicator $t.indicator
    frame $t.interval -bd 1 -relief groove
    IntervalCtl $t.interval
    frame $t.param -bd 1 -relief groove
    ParamCtl $t.param
    frame $t.scenario -bd 1 -relief groove
    ScenarioCtl $t.scenario

    grid $t.indicator -columnspan 2 -sticky news
    grid $t.interval -columnspan 2 -sticky news
    grid $t.scenario $t.param -sticky news
}


# Displays the temperature change due to current parameters

proc DeltaTIndicator {f} {
    global Param Font
    
    label $f.text -text "Global-mean "
    label $f.del -font $Font(greek) -text "D" -padx 0 -bd 0 -highlightthickness 0
    label $f.t -text "T = " -padx 0 -bd 0
    label $f.val -textvariable Param(deltaT) -fg red
    label $f.units -text "deg C" -fg red

    pack $f.text $f.del $f.t $f.val $f.units -side left 
}

proc IntervalCtl {f} {
    global Param Font
    
    scale $f.interval -label "Scenario Year" -from 2000 -to 2100 \
             -orient horizontal -variable Param(interval) -tickinterval 50 \
             -length 180 -sliderlength 54 -command { HackUpdateDT }
    pack $f.interval -side top
}

proc HackUpdateDT {dummy} {
    UpdateDeltaT
}

proc ParamCtl {f} {
    global Param Font
    
    label $f.title -text "MAGICC Setup" -justify center -font $Font(tbtitle)
    radiobutton $f.d -text "Default" -value d -variable Param(gas) \
            -highlightthickness 0 -command {UpdateDeltaT}
    radiobutton $f.u -text "User" -value u -variable Param(gas) \
            -highlightthickness 0 -command {UpdateDeltaT}
    pack $f.title -side top 
    pack $f.d $f.u -side top -anchor w
}

proc ScenarioCtl {f} {
    global Param Font
    
    label $f.title -text "Scenario" -font $Font(tbtitle)
    radiobutton $f.ref -text "$Param(reference) (Ref.)" \
            -variable Param(scenario) -value $Param(reference) \
            -highlightthickness 0 -command {
        UpdateDeltaT
    }
    radiobutton $f.pol -text "$Param(policy) (Pol.)" \
            -variable Param(scenario) -value $Param(policy) \
            -highlightthickness 0 -command {
        UpdateDeltaT
    }
    pack $f.title -side top
    pack $f.ref $f.pol -side top -anchor w
}


# Controls for what kind of data analysis is done.

proc AnalysisWindow {} {
    if {[winfo exists .analysis]} {
        focus .analysis
        raise .analysis
    } else {
        set a [toplevel .analysis]
        wm title .analysis "Analysis"
        
        set f [frame .analysis.main]
        pack $f -side top -fill both -expand on
        AnalysisCtl $f
    }
}


proc AnalysisCtl {f} {
    global Param Font Analyses Annames
    
    set g [frame $f.data -relief groove -bd 1]
    label $g.title -text "Data"
    pack $g.title -side top 

    set g [frame $f.variability -relief groove -bd 1]
    label $g.title -text "Variability"
    pack $g.title -side top

    set g [frame $f.intermodel -relief groove -bd 1]
    label $g.title -text "Intermodel"
    pack $g.title -side top

    foreach a $Analyses {
        set n [lsearch $Analyses $a]

        radiobutton $f.$a -text [lindex $Annames $n] -value $a \
                -variable Param(analysis) -highlightthickness 0 
    }

   pack $f.change $f.error $f.modbase $f.modchange $f.obsbase $f.obschange \
           -side top -anchor w -in $f.data
   pack $f.sdbase $f.sdchange $f.tempsnr  \
           -side top -anchor w -in $f.variability
   pack $f.imsnr $f.probincr  -side top -anchor w -in $f.intermodel

    grid $f.data        -row 1 -column 0 -sticky news -rowspan 2
    grid $f.variability -row 1 -column 1 -sticky news
    grid $f.intermodel  -row 2 -column 1 -sticky news
}


# Variable control (i.e., controls for the variable being mapped).
# Slightly inconvenient that a great many widgets have a -variable
# flag in some form, but it's really the best thing to call it.

proc VariableWindow {} {
    if {[winfo exists .var]} {
        focus .var
        raise .var
    } else {
        set w [toplevel .var]
        wm title .var "Variable"
        
        set f [frame .var.main]
        pack $f -side top -fill both -expand on
        VariableCtl $f
    }
}

proc VariableCtl {f} {

    set f1 [frame $f.f1]
    set f2 [frame $f.f2]

    frame $f1.observable -bd 1 -relief groove
    ObservableCtl $f1.observable
    frame $f1.scaling -bd 1 -relief groove
    ScalingCtl $f1.scaling
    frame $f2.season -bd 1 -relief groove
    SeasonCtl $f2.season
    frame $f2.contour -bd 1 -relief groove
    ContourCtl $f2.contour

    pack $f1 -side left
    pack $f2 -side right -fill y -expand on
    pack $f1.observable $f1.scaling -side top -fill x -expand on
    pack $f2.contour -side top -fill both -expand on
    pack $f2.season -side top -fill both -expand on
}

proc ObservableCtl {f} {
    global Font Observables

    label $f.title -text "Observable"
    pack $f.title -side top
    set i 0
    foreach o $Observables {
        incr i
        radiobutton $f.rb$i -value $o -text $o -variable Param(observable) \
                -highlightthickness 0 -command {set Param(engineflag) true}
        pack $f.rb$i -side top -anchor w
    }
}

proc ScalingCtl {f} {

    label $f.title -text "Scaling"
    pack $f.title -side top
    foreach s {linear exponential} {
        radiobutton $f.$s -text $s -variable Param(scaling) -value $s \
                -highlightthickness 0 -command {set Param(engineflag) true}
        pack $f.$s -side top -anchor w
    }
}

proc SeasonCtl {f} {
    global Param Seasons Font
    
    label $f.title -text "Season:"
    menubutton $f.button -menu $f.button.menu -textvariable Param(seas) \
            -relief raised -borderwidth 1

    set m [menu $f.button.menu -tearoff 0 ]
    for {set i 0} {$i<[llength $Seasons]} {incr i} {
        set s [lindex $Seasons $i]
        $m add radio -label $s -value $s -variable Param(season) \
                -command {set Param(engineflag) true}
        if {$i==11 || $i==15} {$m add separator}
    }
    pack $f.title $f.button -side left
}

proc ContourCtl {f} {

    global Param

    label $f.title -text "Contours"
    radiobutton $f.default -text "Default" -variable Param(contour) \
            -value default -highlightthickness 0
    radiobutton $f.minmax -text "Min/Max" -variable Param(contour) \
            -value minmax -highlightthickness 0

    # The "user" contour will want entry boxes for min and max.
    # However, before Tk v 8.2, validating entries is way too painful,
    # and not validating them is too fragile.  So until we make the
    # upgrade, user contour is disabled.

    # radiobutton $f.user -text "User" -variable Param(contour) \
            -value user -highlightthickness 0

    checkbutton $f.reverse -text "Reverse" -onvalue yes \
            -offvalue no -variable Param(reversecolor)

    pack $f.title $f.default $f.minmax $f.reverse -side top -anchor w
}


# Controls for GCM selection.  Information about the GCMs is defined
# in models.tcl

proc GCMWindow {} {
    if {[winfo exists .gcm]} {
        focus .gcm
        raise .gcm
    } else {
        set w [toplevel .gcm]
        wm title .gcm "Models"
        
        set f [frame .gcm.main]
        pack $f -side top -fill both -expand on
        GCMCtl $f
    }
}       
        

proc GCMCtl {w} {
    global Param gcmPreset gcmList gcmSelection

    set f [frame $w.presets]
    pack $f -side top -fill x
    foreach p [array names gcmPreset] {
        button $f.b$p -text $p -command "UseGCMPreset $p"
        pack $f.b$p -side left -expand on
    }

    radiobutton $f.def1 -text "Def. 1" -variable Param(definition) \
            -value 1 -command {set Param(engineflag) true}
    radiobutton $f.def2 -text "Def. 2" -variable Param(definition) \
            -value 2 -command {set Param(engineflag) true}
    radiobutton $f.def3 -text "Both" -variable Param(definition) \
            -value 3 -command {set Param(engineflag) true}
    pack $f.def3 $f.def2 $f.def1 -side right

    checkbutton $f.aerosol -text "Aerosol effects" \
            -variable Param(aerosol) -command {set Param(engineflag) true}
    pack $f.aerosol -side right

    set f [frame $w.note]
    pack $f -side top
    label $f.note -text "(Right-click models for information.)" \
            -font {Times 10 normal}
    pack $f.note -side left


    set f [frame $w.gcms]
    pack $f -side left
    foreach g $gcmList {
        checkbutton $f.cb$g -text $g -variable gcmSelection($g) \
                -onvalue 1 -offvalue 0 -anchor w -font {Courier 12 normal} \
                -command {set Param(engineflag) true}
        grid $f.cb$g -sticky news -padx 5
        bind $f.cb$g <Button-3> "FullGCMInfo $g"
        # Note: if porting to Mac, will need another binding.
    }

    # let width of presets determine number of columns

    tkwait visibility $f.cb$g
    set w1 [winfo reqwidth $w.presets]
    set w2 [winfo reqwidth $w.gcms]
    set n [llength $gcmList]
    set cols [expr {$w1/$w2}]

    if {$cols > 1} {
        set rows [expr {($n/$cols)+1}]
        set r 0
        set c 0
        foreach g $gcmList {
            grid forget $f.cb$g
            grid $f.cb$g -row $r -column $c -sticky news -padx 5
            incr r
            if {$r >= $rows} {
                set r 0
                incr c
            }
        }
    }
}


proc UseGCMPreset {p} {
    global gcmPreset gcmList gcmSelection Param

    set Param(engineflag) true
    foreach g $gcmList {
        set gcmSelection($g) 0
    }
    foreach g $gcmPreset($p) {
        set gcmSelection($g) 1
    }
}


proc FullGCMInfo {g} {
    
    if [winfo exists .info] {
        focus .info
        raise .info
    } else {
        
        toplevel .info
        wm title .info $g
        wm geometry .info +100+50
        wm geometry .info 600x430
        frame .info.fr 
        
        button .info.ok -text "OK" -width 5 -command {
            destroy .info
        }
        pack .info.ok -side bottom
        pack .info.fr -side top -fill y
        text .info.fr.msg -relief raised -bd 2 -wrap word -bg white \
                -yscrollcommand ".info.fr.scroll set"
        scrollbar .info.fr.scroll -command ".info.fr.msg yview"
        pack .info.fr.scroll -side right -fill y
        pack .info.fr.msg -side left
        focus .info
        
        set infofile "info/$g.txt"
        
        if {[file exists $infofile]} {
            
            set f [open $infofile r]
            while {![eof $f]} {
                .info.fr.msg insert end [read $f 1000]
            }
            close $f
            .info.fr.msg configure -state disabled
        } else {
            .info.fr.msg insert end "No info file exists for this model."
        }
    }
}


proc UpdateDeltaT {} {
    global Param

    set Param(engineflag) true
    
    if {$Param(scenario) == $Param(reference)} {
        set sc "r"
    } else {
        set sc "p"
    }
    
    # Scan in a series of annual temperature changes 

    set magicc_file "../engine/sg$Param(magicc).$Param(gas)$sc"
    set f [open $magicc_file r]
    for {set i 1} {$i <=6} {incr i} {
        gets $f hdr
    }
    for {set i 1} {$i<=140} {incr i} {
        gets $f indata
        scan $indata "%d %f %f %f %f %f" \
                year($i) glt($i) teu($i) tas($i) tr($i) sum($i) 
    }
    close $f
    
    set y [expr $Param(interval)-1960]
    set Param(deltaT) $sum($y)
    set Param(deltaT) [expr {(round($Param(deltaT)*100))/100.}]
}


# Region Selection controls.

proc RegionWindow {} {

    if {[winfo exists .region]} {
        focus .region
        raise .region
    } else {
        set w [toplevel .region]
        wm title .region "Region"
        
        set f [frame .region.main]
        pack $f -side top
        RegionCtl $f
    }
}       

proc RegionCtl {f} {
    global gridsize mapBdy regionCanvas Region regionList

    source regions.tcl

    # Reset mapping parameters so that everything works correctly.

    set mapBdy(n) 90.0
    set mapBdy(e) 180.0
    set mapBdy(s) -90.0
    set mapBdy(w)  -180.0
    set gridsize(can) 8

    set regionCanvas [canvas $f.canvas \
            -width [expr {[MapWidth]+1}] -height [expr {[MapHeight]+1}] \
            -borderwidth 0 -highlightthickness 0 -background white]
    pack $regionCanvas -side left -padx 10 -pady 10


    # The "background" rectangle exists so things can hide behind it.
    $regionCanvas create rect 0 0 [MapWidth] [MapHeight] -fill white -tags bg


    # Aerosol regions.  These are only for display, not for selection
    # as mapping regions.  Number 3 is plotted first so that number 2
    # can overlay it.

    set id [CreateRegion aerosol3]
    $regionCanvas dtag $id region
    $regionCanvas addtag aerosol withtag $id
    $regionCanvas itemconfigure $id -fill #FFFF00 -outline #FFFF00
    LabelRegion aerosol3

    set id [CreateRegion aerosol2]
    $regionCanvas dtag $id region
    $regionCanvas addtag aerosol withtag $id
    $regionCanvas itemconfigure $id -fill #C08000 -outline #C08000
    LabelRegion aerosol2

    set id [CreateRegion aerosol1]
    $regionCanvas dtag $id region
    $regionCanvas addtag aerosol withtag $id
    $regionCanvas itemconfigure $id -fill #ADCB03 -outline #ADCB03
    LabelRegion aerosol1


    # coastlines

    Wldmap load cstwld.dat
    Wldmap draw $regionCanvas 0 0 [MapWidth] [MapHeight] -180 -90 180 90


    # omit graticules until we have internalized the Wldmap routines
    # so that they can be tagged for show/hide.

    #    Wldmap load gratic30.dat
    #    Wldmap draw $regionCanvas 0 0 $width $height -180 -90 180 90


### "In the fullness of time", we plan on using the code below as a
### basis for dealing with mask regions.  However, at the moment I'm
### using a hacked-up interface based on the shell.cfg file that comes
### with the engines.  (Ultimate plan: have engines access a single
### mask file ("mask.dat") that is updated by the GUI.

    # Create mask regions.  The "world" region is also a placeholder
    # for shuffling regions in and out of hiding; shown regions go in
    # front of it, graticules go behind it.  "user" is the region
    # used for click-and-drag selection.  CreateRegion returns the id
    # of the region's label, so we can discard the text for the region
    # by calling "canvas delete" on the result of the function call.

    CreateRegion world
    CreateRegion user

#    foreach r $regionList {
#       CreateRegion $r
#       LabelRegion $r
#    }

    ###  Hack!

# A mask-selection control panel.  This is pretty much a hack; things
# will improve when we can internalize the tile library and
# (hopefully) alter the engine configuration process to use
# GUI-produced maskfiles.


    set masks { A B C D E F G H I J K L M N O P Q R S T U V W X Y 1 2 3 4 5}

    set masknames {Globe Land Ocean NH SH Equ-Pac N3 N3.4 N4 USA Canada \
            Mexico Brazil Africa Europe India China Japan AusNZ C-Asia M-East \
            E-FSU W-FSU ROLA SEASia W-Pac Alaska Grnlnd Antarc Arc-Is}

    set g [frame $f.mask]
    pack $g -side right

    foreach m $masks {
        set mname [lindex $masknames [lsearch $masks $m]]
        radiobutton $g.rb$m -text $mname -variable Param(mask) -value $m \
                -highlightthickness 0 -command {set Param(engineflag) true}
    }

    grid $g.rbA $g.rbK $g.rbU -sticky w -padx 5
    grid $g.rbB $g.rbL $g.rbV -sticky w -padx 5
    grid $g.rbC $g.rbM $g.rbW -sticky w -padx 5
    grid $g.rbD $g.rbN $g.rbX -sticky w -padx 5
    grid $g.rbE $g.rbO $g.rbY -sticky w -padx 5
    grid $g.rbF $g.rbP $g.rb1 -sticky w -padx 5
    grid $g.rbG $g.rbQ $g.rb2 -sticky w -padx 5
    grid $g.rbH $g.rbR $g.rb3 -sticky w -padx 5
    grid $g.rbI $g.rbS $g.rb4 -sticky w -padx 5
    grid $g.rbJ $g.rbT $g.rb5 -sticky w -padx 5
    
    ### End Hack



    # Create selection buttons for the regions defined in regions.tcl;
    # most of these will be associated with maskfiles.

#    set f [frame $f.masks]
#    pack $f -side right
#
#    foreach r "world user $regionList" {
#       radiobutton $f.$r -highlightthickness 0 -command "SelectRegion $r"\
#               -text $Region($r,label) -variable Param(region) -value $r
#       pack $f.$r -side top -anchor w
#    }

radiobutton $g.user -highlightthickness 0 -text "User:" -value Z \
        -variable Param(mask) -command {set Param(engineflag) true}
grid $g.user -sticky w

set h [frame $g.bounds]
grid $h -row 10 -column 1 -columnspan 2
label $h.n -textvariable mapBdy(n) -width 6
label $h.s -textvariable mapBdy(s) -width 6
label $h.e -textvariable mapBdy(e) -width 6
label $h.w -textvariable mapBdy(w) -width 6
label $h.lat -text "Lat:"
label $h.lon -text "Lon:"
label $h.to1 -text "to"
label $h.to2 -text "to" 
    
grid $h.lat $h.s $h.to1 $h.n
grid $h.lon $h.w $h.to2 $h.e


    # bind mouse events

    bind $regionCanvas <Button-1>        {BoxBegin %x %y}
    bind $regionCanvas <B1-Motion>       {BoxDrag  %x %y}
    bind $regionCanvas <ButtonRelease-1> {BoxEnd   %x %y}
    bind $regionCanvas <Button-3>        {BoxClear %x %y}

    SelectRegion user

}

proc CreateRegion {name} {
    global Region regionCanvas

    set n [Lat2Y $Region($name,n)]
    set e [Lon2X $Region($name,e)]
    set s [Lat2Y $Region($name,s)]
    set w [Lon2X $Region($name,w)]

    set id [$regionCanvas create rect $w $n $e $s -outline blue \
            -tags "$name region"]

    # Once we have tcl-ized the tile code, it would be cool to have
    # some code here that grayed in the active blocks in a mask.

    return $id
}

proc LabelRegion {name} {
    global Region regionCanvas

    set n [Lat2Y $Region($name,n)]
    set w [Lon2X $Region($name,w)]

# Something vaguely mysterious here, in that I can't get the "name"
# tags to work.  This is something that you'd need for show/hide
# region toggling.

#    set id [$regionCanvas create text [expr $w + 10] [expr $n + 10] \
#           -anchor w -text $Region($name,label) -tags "$name label"]
    set id [$regionCanvas create text [expr $w + 10] [expr $n + 10] \
            -anchor w -text $Region($name,label) -tags "label"]
    return $id
}

proc SelectRegion {name} {
    global Param regionCanvas Region mapBdy

    $regionCanvas itemconfigure region -outline blue
    set Param(region) $name
    $regionCanvas itemconfigure $Param(region) -outline red

    foreach d {n e s w} {
        set mapBdy($d) $Region($name,$d)
    }
}


# The Box* functions are used for click-and-drag region selection.

proc BoxBegin {x y} {
    global mapBdy gridsize boxAnchor boxLast regionCanvas Param

    set mapBdy(n) 90.0
    set mapBdy(e) 180.0
    set mapBdy(s) -90.0
    set mapBdy(w) -180.0

    set boxAnchor(lon) [CenterLon [X2Lon $x]]
    set boxAnchor(lat) [CenterLat [Y2Lat $y]]

    set boxLast(lon) $boxAnchor(lon)
    set boxLast(lat) $boxAnchor(lat)

    $regionCanvas delete user
    DrawBox
    SelectRegion user
    set Param(mask) Z
    set Param(engineflag) true
}

proc DrawBox {} {
    global gridsize boxAnchor boxLast regionCanvas

    if {$boxAnchor(lon) < $boxLast(lon)} {
        set east [Lon2X [expr $boxLast(lon) + $gridsize(map)/2.0]]
        set west [Lon2X [expr $boxAnchor(lon) - $gridsize(map)/2.0]]
    } else {
        set west [Lon2X [expr $boxLast(lon) - $gridsize(map)/2.0]]
        set east [Lon2X [expr $boxAnchor(lon) + $gridsize(map)/2.0]]
    }

    if {$boxAnchor(lat) < $boxLast(lat)} {
        set north [Lat2Y [expr $boxLast(lat) + $gridsize(map)/2.0]]
        set south [Lat2Y [expr $boxAnchor(lat) - $gridsize(map)/2.0]]
    } else {
        set south [Lat2Y [expr $boxLast(lat) - $gridsize(map)/2.0]]
        set north [Lat2Y [expr $boxAnchor(lat) + $gridsize(map)/2.0]]
    }

    $regionCanvas create rect $west $north $east $south -outline red -tags "user region"
}


proc BoxDrag {x y} {
    global boxLast regionCanvas

    set boxLast(lon) [CenterLon [X2Lon $x]]
    set boxLast(lat) [CenterLat [Y2Lat $y]]

    $regionCanvas delete user
    DrawBox
}

proc BoxEnd {x y} {
    global boxAnchor boxLast mapBdy gridsize

    if {$boxAnchor(lon) < $boxLast(lon)} {
        set mapBdy(e) [expr $boxLast(lon) + $gridsize(map)/2.0]
        set mapBdy(w) [expr $boxAnchor(lon) - $gridsize(map)/2.0]
    } else {
        set mapBdy(w) [expr $boxLast(lon) - $gridsize(map)/2.0]
        set mapBdy(e) [expr $boxAnchor(lon) + $gridsize(map)/2.0]
    }

    if {$boxAnchor(lat) < $boxLast(lat)} {
        set mapBdy(n) [expr $boxLast(lat) + $gridsize(map)/2.0]
        set mapBdy(s) [expr $boxAnchor(lat) - $gridsize(map)/2.0]
    } else {
        set mapBdy(s) [expr $boxLast(lat) - $gridsize(map)/2.0]
        set mapBdy(n) [expr $boxAnchor(lat) + $gridsize(map)/2.0]
    }
}


proc BoxClear {x y} {
    global mapBdy regionCanvas Param
    set mapBdy(n) 90.0
    set mapBdy(e) 180.0
    set mapBdy(s) -90.0
    set mapBdy(w) -180.0

    $regionCanvas delete user
    SelectRegion world
    set Param(mask) A
    set Param(engineflag) true
}


proc Lon2X {lon} {
    global mapBdy gridsize
    set x [expr {int(($lon - $mapBdy(w)) * $gridsize(can) / $gridsize(map))}]
}

proc Lat2Y {lat} {
    global mapBdy gridsize
    set y [expr {int(($mapBdy(n) - $lat) * $gridsize(can) / $gridsize(map))}]
}

proc X2Lon {x} {
    global mapBdy gridsize
    set lon [expr {$mapBdy(w) + $x * $gridsize(map) / $gridsize(can)}]
}

proc Y2Lat {y} {
    global mapBdy gridsize
    set lat [expr {$mapBdy(n) - $y * $gridsize(map) / $gridsize(can)}]
}


# Currently, we are using mask files that are defined on the whole
# globe, so we don't want to resize the map for user-defined regions.
# Therefore MapWidth and MapHeight need to return constant values.

proc MapWidth {} {
    global mapBdy
#    return [expr {[Lon2X $mapBdy(e)] - [Lon2X $mapBdy(w)]}]
    return [expr {[Lon2X 180] - [Lon2X -180]}]
}


# WHOA!  This cannot be right.  However, it currently works, and if I
# change it to what it should be, it doesn't work.  So I will leave it
# for now, until further investigation reveals the true solution.  (I
# suspect there's something wrong with Lat2Y)

proc MapHeight {} {
    global mapBdy
#    return [expr {[Lon2X $mapBdy(n)] - [Lon2X $mapBdy(s)]}]
    return [expr {[Lon2X 90] - [Lon2X -90]}]
}

proc CenterLon {lon} {
    global gridsize
    return [expr {$gridsize(map)*(floor($lon/$gridsize(map))+0.5)}]
}

proc CenterLat {lat} {
    global gridsize
    return [expr {$gridsize(map)*(floor($lat/$gridsize(map))+0.5)}]
}


# PutMap draws the blank world map with the word "SCENGEN" across it
# in light gray.  It gets called when a new map window is created.

proc PutMap {} {
    global can Font mapBdy

    $can delete all
    $can create rectangle 0 0 [MapWidth] [MapHeight] -fill white -outline black
    $can create text [Lon2X -110] [Lat2Y 0] -text "SCENGEN" \
            -font $Font(sgtitle) -anchor w -fill #dddddd

    Wldmap load cstwld.dat

    # map is constant size; see MapWidth, etc. for explanation
#    Wldmap draw $can 0 0 [MapWidth] [MapHeight] \
#           $mapBdy(w) $mapBdy(s) $mapBdy(e) $mapBdy(n)
    Wldmap draw $can 0 0 [MapWidth] [MapHeight] \
            -180 -90 180 90
}


proc ViewMap {} {
    global flagview Param map gcmList gcmSelection
    
    if {![winfo exists $map]} {
        MapWindow
    }

    set flag 1    
    foreach g $gcmList {if {$gcmSelection($g)} {set flag 0} }
    if { ($flag) && ($Param(analysis) != "obsbase") && !($Param(aerosol)) } {
        Error "Select at least one GCM or turn 'Aerosol effects' on."
    } else {

        Msg "Running!"
        update 
        InvokeEngine
        set Param(engineflag) false
        Display
        destroy .msg
    }
}      


proc InvokeEngine {} {
    global Param

    # check to see if we actually need to rerun the data, and if so then:

    if {$Param(engineflag)} {

        set here [pwd]
        cd ../engine
        WriteConfigFile
        exec intermod.exe
        exec standdev.exe
        cd $here
        if {$Param(mask) == "Z"} {WriteMaskFile}
    }
}


# Note the leading space before each config entry line.  Also,
# Param(mask) has to be the eighth character on the line.

proc WriteConfigFile {} {
    global Param gcmList gcmSelection Seasons mapBdy

    set f [open shell.cfg w]
    puts $f " Config file written by SCENGEN GUI"
    puts $f " $mapBdy(s)"
    puts $f " $mapBdy(n)"
    puts $f " $mapBdy(w)"
    puts $f " $mapBdy(e)"
    if {$Param(mask) == "Z"} { 
        puts $f " 0" 
        puts $f "       A"
    } else {
        puts $f " 1"
        puts $f "       $Param(mask)"
    }
    if {$Param(obs) == "temp"} {puts $f " 1"} else {puts $f " 2"}
    puts $f " [expr [lsearch -exact $Seasons $Param(season)]+1]"
    if {$Param(scaling) == "linear"} {puts $f " 1"} else {puts $f " 2"}
    if {$Param(gas) == "d"} {puts $f " 1"} else {puts $f " 2"}
    if {$Param(scenario) == $Param(reference)} {puts $f " 1"} else {puts $f " 2"}
    puts $f " $Param(aerosol)"
    puts $f " $Param(interval)"

    set n 0
    set gcms {}
    foreach g $gcmList {
         if {$gcmSelection($g)} {
             incr n
             lappend gcms [expr [lsearch -exact $gcmList $g]+1]
         }
    }

    puts $f " $n"
    puts $f " $Param(definition)"
    puts $f " [join $gcms "\n "]"
    close $f
}


proc WriteMaskFile {} {
    global mapBdy

    set f [open "maskfiles/MASK.Z" w]

    for {set lati 0} {$lati < 36} {incr lati} {
        for {set loni 0} {$loni < 72} {incr loni} {
            set lon [expr {-177.5 + 5*$loni}]
            set lat [expr {87.5 - 5*$lati}]
            if { $lat < $mapBdy(n) && $lat > $mapBdy(s) && \
                    $lon < $mapBdy(e) && $lon > $mapBdy(w)} {
                puts -nonewline $f "1" 
            } else {
                puts -nonewline $f "0" 
            }
        }
        puts $f ""
    }
    close $f
}
    

# Display actually draws the map on the screen.

proc Display {} {

    global can Param tcl_platform map Seasons Font gcmList gcmSelection
    global Contour contourPrecision contourColor
    global xgrid ygrid xs lat1 lat2 lon1 lon2 startx starty xsize ysize
    global mapBdy gridsize Analyses Anphrases Anpreps
    
    set x 72
    set y 36
    set size 8
    set cindex "$Param(obs),$Param(analysis)"

    set maskfile world.msk
    set cstfile cstwld.dat
    set dir "../engine/scengen"
    switch $Param(analysis) {
        change     {set datafile "$dir/DEL2USE.OUT"}
        error      {set datafile "$dir/ERROR.OUT"}
        imsnr      {set datafile "$dir/IM-SNR.OUT"}
        modbase    {set datafile "$dir/MODBASE.OUT"}
        modchange  {set datafile "$dir/ABS-MOD.OUT"}
        obsbase    {set datafile "$dir/OBSBASE.OUT"}
        obschange  {set datafile "$dir/ABS-OBS.OUT"}
        probincr   {set datafile "$dir/PROBINCR.OUT"}
        sdbase     {set datafile "$dir/BASE-SD.OUT"}
        sdchange   {set datafile "$dir/DELTA-SD.OUT"}
        tempsnr    {set datafile "$dir/BAROFSNR.OUT"}
    }


### Note: There's lots of code here that deals with plotting
### rectangular sub-regions of the map.  (Kludgey because of tile
### library limitations.)  At the moment, however, we're dealing with
### masks that are all defined across the entire globe.  To make the
### "user-defined" mask display properly, we reset the mapBdys.  Yes,
### this means that the mask region resets to global every time you
### press "SHOW".  Eit.

    set mapBdy(n) 90
    set mapBdy(e) 180
    set mapBdy(s) -90
    set mapBdy(w) -180

    # (1) set up map outline

    $can delete all
    $can create rectangle 0 0 [MapWidth] [MapHeight] -fill white -outline black

    # (2) make color field: read tile values, create tiles, then color them.

    Tile_missing 9999
    Tile_read $datafile $x $y
    Tile_mask "maskfiles/MASK.$Param(mask)"

    if {$Param(mask) != "A"} {
        set Param(mask) A
        set Param(engineflag) true
    }

    ### WARNING: THIS IS A MASSIVE KLUDGE ###

    # The Tile library doesn't support plotting a rectangular subset
    # of the data that it has read in.  (Yet another reason why the
    # functions therein need to be subsumed into this program once
    # we've migrated to v 8.0+) So what I do is offset it so that it
    # lines up with the subregion map, plot the whole color field, and
    # then hide the extra bits behind big gray blocks.  Geh.  Ugly.

    set x0 [expr {int((-180 - $mapBdy(w)) * $gridsize(can) / $gridsize(map))}]
    set y0 [expr {int(($mapBdy(n)- 90) * $gridsize(can) / $gridsize(map))}]
    Tile_draw $can $gridsize(can) $gridsize(can) $contourPrecision($cindex) $x0 $y0
    $can create rect $x0 [Lat2Y $mapBdy(s)] [Lon2X 180] [Lat2Y -90] -fill gray -outline gray
    $can create rect [Lon2X $mapBdy(e)] $y0 [Lon2X 180] [Lat2Y -90] -fill gray -outline gray
    $can create rectangle 0 0 [MapWidth] [MapHeight] -outline black

    ### end massive kludge ###


    ColorGrid


    # (3) plot coastlines

    Wldmap load $cstfile
    Wldmap draw $can 0 0 [MapWidth] [MapHeight] $mapBdy(w) $mapBdy(s) $mapBdy(e) $mapBdy(n)


    # (4) create graticules

    # graticules are implemented using Wldmap because that way we can
    # take advantage of the fact that it already does all the map
    # clipping and scaling stuff and define the graticules in lat-lon
    # format.  Two graticule files have been created: 30-degree and
    # 5-degree.  5-degree would also be useful for a "show grid"
    # option when doing region selection.

    Wldmap load gratic30.dat
    Wldmap draw $can 0 0 [MapWidth] [MapHeight] $mapBdy(w) $mapBdy(s) $mapBdy(e) $mapBdy(n)


    # (5) bind dynamic annotations

    set ymin [expr [Tile_min]]
    set ymax [expr [Tile_max]]
    set ymin [format "%1.1f" $ymin]
    set ymax [format "%1.1f" $ymax]

    bind $can <Enter> {ShowVal %x %y}
    bind $can <Leave> {ClearVal}
    bind $can <Motion> {ShowVal %x %y}


    # (6) do static annotations (they're canvas objects so they'll print)

    set index [lsearch -exact $Analyses $Param(analysis)]
    set title "[lindex $Anphrases $index]"
    set title "$title [lindex $Anpreps $index]"
    set title "$title $Param(season) $Param(observable)"
    $can create text [expr [MapWidth]+10] 10 -text $title \
            -width 190 -anchor nw

    switch $Param(analysis) {
        error -
        obsbase -
        modbase -
        sdbase { }
        default {
            $can create text [expr [MapWidth]+10] 50 -anchor w \
                    -text "Global-mean dT: $Param(deltaT) deg C"
            $can create text [expr [MapWidth]+10] 70 -anchor w \
                    -text "Year $Param(interval), Scenario $Param(scenario)"
        }
    }
    if {$Param(analysis) != "obsbase"} {
        $can create text [expr [MapWidth]+10] 90 -anchor w -text "Models:"
        set gcms {}
        foreach g $gcmList {if {$gcmSelection($g)} {lappend gcms $g}}
        $can create text [expr [MapWidth]+10] 110 -anchor nw -width 100 -text $gcms
    }

    $can create text [expr [MapWidth]+80] 270 -anchor w -text "Range: $ymin to $ymax"
    DrawColorbar $can [expr [MapWidth]+140] 90
}


# Hemi formats a signed latitude or longitude value to a string with
# hemisphere notation and a degree symbol.

proc Hemi { x n s } {
    if {$x==0} {
        return "0\xb0"
    } else {
        if {$x<0} {
            return "[expr -$x]\xb0$s"
        } else {
            return "$x\xb0$n"
        }
    }
}


# ClearVal and ShowVal update the displayed latitude, longitude, and
# map value associated with the current mouse location.

proc ClearVal {} {
    global Lat Lon Val map
    set Lat ""
    set Lon ""
    set Val ""
}


proc ShowVal { mx my } {
    global mapBdy gridsize 
    global Lat Lon Val Param

    set x [expr {int(floor($mx/$gridsize(can)))}]
    set y [expr {int(floor($my/$gridsize(can)))}]

    set lon [X2Lon $mx]
    set lat [Y2Lat $my]

    if {$lon >= $mapBdy(e) || $lon <= $mapBdy(w) || \
        $lat >= $mapBdy(n) || $lat <= $mapBdy(s)} {
        ClearVal
    } else {
        set north [expr {[CenterLat $lat] + $gridsize(map)/2.0}]
        set south [expr {[CenterLat $lat] - $gridsize(map)/2.0}]
        set east [expr {[CenterLon $lon] + $gridsize(map)/2.0}]
        set west [expr {[CenterLon $lon] - $gridsize(map)/2.0}]

        set Lat "[Hemi $south N S] - [Hemi $north N S]"
        set Lon "[Hemi $west E W] - [Hemi $east E W]"
        if {([Tile_land $x $y] && [Tile_value $x $y 1]!=9999)} {
            set Val "[Tile_value $x $y 1] $Param(units)"
        } else {
            set Val ""
        }
    }
}


# Draws a colorbar on $can at $x0 $y0.  Note that the Contour variable
# gets updated by ColorGrid, which is currently always called before
# DrawColorBar is called; if that changes, it should be updated here,
# too.

proc DrawColorbar {can x0 y0} {

    global Param
    global Contour contourColor

    $can delete cbar

    set h 14
    set w 16
    set x1 $x0
    set x2 [expr $x0 + $w]
    
    for {set i 0} {$i < 11} {incr i} {
        set y1 [expr {$y0+(10-$i)*15}]
        set y2 [expr {$y1+$h}]
        if {$Param(reversecolor)} {
        $can create rectangle $x1 $y1 $x2 $y2 -fill $contourColor([expr 10-$i]) -tag cbar
        } else {
        $can create rectangle $x1 $y1 $x2 $y2 -fill $contourColor($i) -tag cbar
        }
    }
    
    # labels:
    set xl [expr {$x0 + 50}]
    $can create text $xl [expr $y0 - 10] -text $Param(units) -anchor e
    for {set i 0} {$i<10} {incr i} {
        set ypos [expr {$y0+15*(10-$i)}]
        set val [format "%4.2f" [lindex $Contour $i]]
        $can create text $xl $ypos -anchor e -tag cbar -text $val 
    }
}


# The values layer of the map is composed of tiles that have been
# tagged with their z-values; the map gets colored in by ColorGrid
# configuring the tiles with fill colors in the $contourColor array,
# defined in the user-configurable file contours.tcl.

proc ColorGrid {} {
    
    global can Contour contourColor contourPrecision Param

    # Synch the Contour variable with current settings just before use.
    SynchContour

    # We use Tile_min and _max to make sure we color all the tiles.
    # It's possible that Tile_min is greater than the minimum contour
    # value; that's okay, it still Does The Right Thing.

    set p $contourPrecision($Param(obs),$Param(analysis))

    set fencepost(0) [expr {int(floor([Tile_min]*$p)-1)}]
    set fencepost(11) [expr {int(ceil([Tile_max]*$p)+1)}]
    for {set i 0} {$i < 10} {incr i} {
        set fencepost([expr {$i+1}]) [expr {int(floor([lindex $Contour $i] * $p))}]
    }

    for {set i 0} {$i <= 10} {incr i} {
        if {$Param(reversecolor)} {
            set color $contourColor([expr {10-$i}])
        } else {
            set color $contourColor($i)
        }

        for {set j $fencepost($i)} {$j < $fencepost([expr $i+1])} {incr j} {
            $can itemconfigure d$j -fill $color
        }
    }
}


# Synchs the $Contour variable with the display settings.  Depends on
# a large number of variables, so we call it from ColorGrid (just
# before it's used) rather than watching variables.

proc SynchContour {} {
    global Contour Param contourDefaults

    switch $Param(contour) {
        default {
            set Contour $contourDefaults($Param(obs),$Param(analysis))
        }
        minmax {
            ConstructContour [format "%4.2f" [Tile_min]] [format "%4.2f" [Tile_max]]
        }
        user {
            Error "User-defined contours not implemented yet.  Using default."
            set Contour $contourDefaults($Param(obs),$Param(analysis))
        }
    }
}


proc ConstructContour {min max} {

    global Contour contourPrecision Param

    set min [Tile_min]
    set max [Tile_max]
    set step [expr {($max-$min)/11}]

    set p [expr 1.0/$contourPrecision($Param(obs),$Param(analysis))]
    if {$step < $p} {
        Error "Warning! Contour spacing is less than data precision."
    }
        
    set minmax {}
    for {set i 0} {$i < 10} {incr i} {
        lappend minmax [expr {$min + $step * ($i + 1)}] 
    }
    set Contour $minmax
}
        
### We will want to add a "save data" function, but don't have user
### requirements for it yet.  On the other hand, an advanced user can
### just grab the .OUT files from ../engine/imout or sdout, so it may
### be unnecessary.

### Printing does not currently work.  Eit!  Note: When it gets fixed,
### we want to add a selection control that lists the existing maps
### and asks which one to print.  Shouldn't be too hard, actually.

proc PrintMap {} {
    global flagview Font Param

    if {0} {
        # was "if flagview = 0"
        Error "View map before saving image"    
    } elseif [winfo exists .save] {
        raise .save
        focus .save
    } else {
        set tl [toplevel .save]
        wm title $tl "Save Image"
        wm geometry $tl +200+150
        set f [frame $tl.f]
        pack $f -side bottom 
        button $f.ok -text "OK" -width 6 -command {
            set outfile "output/$Param(outname).eps"
            if {[file exists $outfile]} {
                if {[info exists clobbercheck]} {unset clobbercheck}
                VerifyClobber $outfile
                tkwait variable clobbercheck
                if {$clobbercheck} {
                    $can postscript -file "output/$Param(outname).eps" -rotate yes -fontmap Fontmap
                    destroy .save
                } else {
                    set Param(outname) ""
                }
            } else {
                $can postscript -file "output/$Param(outname).eps" -rotate yes -fontmap Fontmap
                destroy .save
            }           
        }
        button $f.cancel -text "Cancel" -command {
            destroy .save
        }
        pack $f.ok -side left -padx 3
        pack $f.cancel -side right -padx 3 
        
        message $tl.msg -text "Enter root file name (.eps \
                extension will be added automatically)." \
                -font $Font(message) -justify left -width 300 -fg blue
        pack $tl.msg -side top
        label $tl.lb -text "File name: "
        entry $tl.entry -width 30 -relief sunken -bd 2 -textvariable Param(outname)
        pack $tl.lb $tl.entry -side left -padx 1m -pady 2m -fill x
    }
}


proc VerifyClobber {filename} {
    global Font clobbercheck

    set tl [toplevel .exists]
    wm title $tl "File Exists"
    wm geometry $tl +200+220
    set f [frame $tl.f]
    pack $f -side bottom
        
    button $f.yes -text "Yes" -width 6 -command {
        set clobbercheck 1
        destroy .exists
    }

    button $f.cancel -text "Cancel" -command {
        set clobbercheck 0
        destroy .exists
        focus .save
        raise .save
    }
        
    pack $f.yes -side left -padx 3
    pack $f.cancel -side right -padx 3
    
    set f [frame $tl.top]
    pack $f -side top
    label $f.l -bitmap warning -fg red
    message $f.msg -text "File $filename already exists! \
            Do you want to overwrite existing file?" -font $Font(message) \
            -justify left -width 300
    pack $f.l $f.msg -side left -pady 5
    grab $tl
}


# SGExit does a verify popup before killing the program.

proc SGExit {} {
    
    set tl [toplevel .sgexit]
    wm title $tl "Exit SCENGEN"
    wm geometry $tl 250x80+280+180
    label $tl.l1 -text "This will end your SCENGEN session!" -fg blue
    pack $tl.l1 -side top -padx 10 -pady 5
    set f [frame $tl.buttons -bd 10]
    pack $f -side top 
    button $f.ok -text "OK" -width 6 -command exit
    button $f.cancel -text "Cancel" -command "destroy $tl"
    pack $f.ok -side right -padx 3 -pady 0
    pack $f.cancel -side left -padx 3 -pady 0
    grab $tl
}


# Error handler popup.

proc Error {txt} {
    
    bell
    toplevel .err 
    wm title .err "Error"
    wm geometry .err +280+180
    wm minsize .err 200 80
    label .err.lb -text $txt -padx 4m -pady 3m -wraplength 150
    button .err.but -text "OK" -width 5 -command {destroy .err}
    pack .err.lb .err.but -side top
    grab .err
}

# transient "Running" popup message

proc Msg {message} {
    
    toplevel .msg 
    wm title .msg " "
    wm geometry .msg +280+180
    wm minsize .msg 200 50
    label .msg.lb -text $message -padx 4m -pady 3m -fg blue
    pack .msg.lb -side top
    grab .msg
    #    button .msg.ok -text "OK" -command {destroy .msg}
#    pack .msg.ok -side top
}
