# This is contours.tcl, which defines information about the contours
# SCENGEN uses to generate its colormap.  

# contourReverseColor is a toggle for red-blue = hi-low ("no") or low-hi ("yes")

set contourReverseColor(temp,change)    no
set contourReverseColor(temp,error)     no
set contourReverseColor(temp,imsnr)     no
set contourReverseColor(temp,modbase)   no
set contourReverseColor(temp,modchange) no
set contourReverseColor(temp,obsbase)   no
set contourReverseColor(temp,obschange) no
set contourReverseColor(temp,probincr)  no
set contourReverseColor(temp,sdbase)    no
set contourReverseColor(temp,sdchange)  no
set contourReverseColor(temp,tempsnr)   no

set contourReverseColor(prec,change)    yes
set contourReverseColor(prec,error)     no
set contourReverseColor(prec,imsnr)     yes
set contourReverseColor(prec,modbase)   yes
set contourReverseColor(prec,modchange) yes
set contourReverseColor(prec,obsbase)   yes
set contourReverseColor(prec,obschange) yes
set contourReverseColor(prec,probincr)  yes
set contourReverseColor(prec,sdbase)    yes
set contourReverseColor(prec,sdchange)  no
set contourReverseColor(prec,tempsnr)   yes


# contourMin and contourMax are the extrema for the user-defined contours.

set contourMin 0
set contourMax 0


# A contour is a 10-element list of contour boundary values.  Any map
# tile with a z-value in the range [contour(i), contour(i+1) ) is
# colored with color(i).  The data have a precision of one part in
# $contourPrecision.  In other words, if the data have two digits of
# precision (x = 0.05), $contourPrecision is 100.  x = 0.1,
# $contourPrecision = 10.  And so on.  The contourDefaults array has
# default contouring intervals for various combinations of data type
# and analysis type.  Note that these arrays use the Tcl composite
# indexing hack -- "temp,Change" is constructed from two sub-indices,
# but as far as Tcl is concerned it's a single associative array
# string index, so it's critical that there be no whitespace in the
# index.

set contourPrecision(temp,change)    100
set contourPrecision(temp,error)     10
set contourPrecision(temp,imsnr)     10
set contourPrecision(temp,modbase)   10
set contourPrecision(temp,modchange) 10
set contourPrecision(temp,obsbase)   10
set contourPrecision(temp,obschange) 10
set contourPrecision(temp,probincr)  100
set contourPrecision(temp,sdbase)    100
set contourPrecision(temp,sdchange)  10
set contourPrecision(temp,tempsnr)   10

set contourDefaults(temp,change)    {-0.5 0.0 0.5 1.0 1.5 2.0 2.5 3.0 3.5 4.5}
set contourDefaults(temp,error)     {-9 -7 -5 -3 -1 1 3 5 7 9}
set contourDefaults(temp,imsnr)     {0.0 1.0 2.0 3.0 4.0 5.0 6.0 7.0 8.0 9.0}
set contourDefaults(temp,modbase)   {-25 -15 -10 -5 0 5 10 15 20 25}
set contourDefaults(temp,modchange) $contourDefaults(temp,modbase)
set contourDefaults(temp,obsbase)   $contourDefaults(temp,modbase)
set contourDefaults(temp,obschange) $contourDefaults(temp,modbase)
set contourDefaults(temp,probincr)  {0.80 0.82 0.84 0.86 0.88 0.90 0.92 0.94 0.96 0.98}
set contourDefaults(temp,sdbase)    {0.2 0.4 0.6 0.8 1.0 1.2 1.4 1.6 1.8 2.0}
set contourDefaults(temp,sdchange)  {-15 -10 -5 0 5 10 15 20 25 30}
set contourDefaults(temp,tempsnr)   $contourDefaults(temp,imsnr)

set contourPrecision(prec,change)    10
set contourPrecision(prec,error)     1
set contourPrecision(prec,imsnr)     10
set contourPrecision(prec,modbase)   10
set contourPrecision(prec,modchange) 10
set contourPrecision(prec,obsbase)   10
set contourPrecision(prec,obschange) 10
set contourPrecision(prec,probincr)  100
set contourPrecision(prec,sdbase)    100
set contourPrecision(prec,sdchange)  1
set contourPrecision(prec,tempsnr)   10

set contourDefaults(prec,change)    {-9 -6 -3 0 3 6 9 12 15 18}
set contourDefaults(prec,error)     {-90 -70 -50 -30 -10 10 30 50 70 90}
set contourDefaults(prec,imsnr)     {-1.0 -0.5 0.0 0.5 1.0 1.5 2.0 2.5 3.0 3.5}
set contourDefaults(prec,modbase)   {0.5 1.0 2.0 3.0 4.0 5.0 6.0 7.0 8.0 10.0}
set contourDefaults(prec,modchange) $contourDefaults(prec,modbase)
set contourDefaults(prec,obsbase)   $contourDefaults(prec,modbase)
set contourDefaults(prec,obschange) $contourDefaults(prec,modbase)
set contourDefaults(prec,probincr)  {0.05 0.10 0.15 0.20 0.40 0.60 0.80 0.85 0.90 0.95}
set contourDefaults(prec,sdbase)    {0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0}
set contourDefaults(prec,sdchange)  {-15 -10 -5 0 5 10 15 20 25 30}
set contourDefaults(prec,tempsnr)   $contourDefaults(prec,imsnr)


# initialize to default:

set Contour $contourDefaults(temp,change)


# The colors used for filling contours.  Color(1) is used for values
# between Contour(0) and Contour(1).  Color(0) is for values less than
# Contour(0).  Color(10) is for values greater than or equal to
# Contour(9).

set contourColor(10)  #a63200    ;#dark brown
set contourColor(9)   #ff0000    ;#red
set contourColor(8)   #ffb4b4    ;#light pink
set contourColor(7)   #ff7800    ;#orange
set contourColor(6)   #ffff00    ;#yellow
set contourColor(5)   #00ff00    ;#light green
set contourColor(4)   #008787    ;#teal
set contourColor(3)   #00ffff    ;#light blue
set contourColor(2)   #0087ff    ;#mid blue
set contourColor(1)   #9300c3    ;#dark purple
set contourColor(0)   #ff00ff    ;#magenta














